
c with diagnostic writes for (il=67,ij=02,ik=1,2,3) and ic = ?? (rmoh)
c 8-10-10
c=============================================================================
c
c $Id: drydep_update.F,v 1.4 2002/05/20 16:44:09 xhliu Exp $
c
c CODE DEVELOPER
c   Dan Bergmann, LLNL
c   dbergmann@llnl.gov
c
c  Modified 4/28/08:  Sandy Sillman,UMich (see ADDED: Sillman...)
c   sillman@umich.edu
c
c FILE
c   drydep_update.F
c
c ROUTINES
c   Update_Drydep
c   Depvel
c   Canopy_Resistance
c   Surface_Resistance
c
c=============================================================================
c drydep_updateVDEP2.F  zero protect added
c drydep_Update.F
c Modified version (Sillman, 2008) to account for vertical diffusion
c   in the calculation of dry deposition, based on diffu_dao2.F
c
c This version uses vertical diffusion to determine dry deposition removal,
c  but it does not calculate vertical diffusion itself.
c 
c This version requires a change in imp_step.F only to include 
c  the added arguments for Update_Drydep
c
c TEST WRITE for conservation of mass - commented out.
c
c Note: The hstar and delH_298_over_R enter from: setkin_depos.h
c
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Update_Drydep
c
c DESCRIPTION
c   This routine updates const based on dry deposition using dry deposition
c   velocities calculated by the Harvard dry deposition model.
c
c Expansion 4/28/08:  This routine also incorporates the vertical diffusion
c   algorithm from diffu_dao2.F (Do_Vert_Diffu)
c   which calculates vertical diffusion using an implicit technique.
c   This is used to calculate the effect of dry deposition on all 
c    vertical layers, but not to calculate vertical diffusion itself.
c 
c   (A combined solution for dry deposition and vertical mixing 
c     is also possible, using the same format as this,
c     but it would be problematic because dry deposition would 
c     occur following chemistry and emissions but before convection.)
c
c   Note:  Constituent mass is conserved during mixing.
c          Constant field should remain constant. 
c                             - except for dry dep loss. (tested)
c
c ARGUMENTS
c    Update_Dry_Deposition 
c   pr_dry_depos  : should dry depositions be written to periodic const
c                   output file?
c   lwi_flags : array of flags that indicate land, water, or ice
c   ai        : pressure = (ai * pt) + (bi * psf), ai at zone interface
c   bi        : pressure = (ai * pt) + (bi * psf), bi at zone interface
c   am        : pressure = (am * pt) + (bm * psf), am at zone midpoint
c   bm        : pressure = (am * pt) + (bm * psf), bm at zone midpoint
c   mcor      : area of grid box  (m^2)
c   latdeg    : latitude  (deg)
c   londeg    : longitude (deg)
c   humidity  : specific humidity (g/kg)
c   max_cloud : maximum overlap cloud fraction for LW
c   pbl       : planetary boundary layer depth (m)
c   radswg    : net downward shortwave radiation at ground (W/m^2)
c   ran_cloud : random  overlap cloud fraction for LW
c   surf_air_temp : surface air temperature (degK)
c   surf_rough    : surface roughness (m)
c   ustar     : friction velocity (m/s)
c   mass      : total mass of the atmosphere within each grid box   (kg)
c   psf       : surface pressure field at t1, known at zone centers (mb)
c   kel1      : temperature field at t1, known at zone centers (degK)
c   const     : species concentration, known at zone centers (mixing ratio)
c   dry_depos : dry deposition accumulated since last output (kg/m^2)
c   diffaer   : aerosol diffusivity at bottom layer (m^2/s)
c   s_radius  : aerosol radius at bottom layer (m)
c   s_velocity: aerosol settling velocity at bottom layer (m/s)
c
c ADDED ARGUMENTS from diffu_dao2.F  (Sillman 4/27/08)
c   kzz     : array of vertical diffusion coefficients (m^2/s)
c   press3c : atmospheric pressure at the center of each grid box (mb)
c   press3e : atmospheric pressure at the edge   of each grid box (mb)
c
c Note: arguments from diffu_dao2.F already included in drydep_update.F:
c
c   tdt     : model time step (s) - may be included from 'include',
c                since it is used in Update_Drydep. in imp_control.h.
c   vert_diffu_coef : scalar vertical diffusion coefficient (m^2/s) 
c                included in imp_control.h
c   pbl     : planetary boundary layer depth (m)
c   psf     : surface pressure field at t1, known at zone centers (mb)
c   qq      : species concentration (mixing ratio) -> =const 
c   advec_flag : array of flags that indicate whether or not to advect a
c                given species -> from imp_control.h
c 
c
c-----------------------------------------------------------------------------

      subroutine Update_Drydep
     &  ( lwi_flags, mcor, latdeg,
     &   londeg,  max_cloud,  radswg, ran_cloud,
     &   surf_air_temp, surf_rough, ustar, mass, psf,  const,
     &   dry_depos, diffaer, s_radius, s_velocity
c ADDED: Sillman 4/28/08
     &        , kzz, press3c, press3e,
     &   tdt, box_zm, box_zi, drydep_vel, ireg, iland, iuse, xlai)

! edited by Guangxing Lin
! remove pr_dry_depos, ai, bi, am, bm, humidity,kel1, pbl
! added tdt, box_zm, box_zi,drydep_vel, ireg, iland,iuse, xlai
      use h_imp_phys_constants
      use h_imp_time_constants
      use h_imp_dims, only: i1_gl, i2_gl,ju1_gl,j2_gl, ilo,ihi,julo,jhi,
     &k1,k2,i1,i2, ju1,j2, ilong, num_species
      use h_setkin_par
      use h_setkin_depos
      use h_imp_control,   only: nhms, nymd, advec_flag,
     &    diffu_opt,vert_diffu_coef
      use h_setkin_mw,     only: mw => mw_data
      use h_imp_emiss_constants 
      implicit none
 
!#     include "imp_emiss_constants.h"
!#     include "imp_phys_constants.h"
!#     include "imp_time_constants.h"
!#     include "imp_dims.h"
!#     include "imp_control.h"
!#     include "imp_emiss_harvard.h"
ccc!!
!#     include "setkin_par.h"
!#     include "setkin_depos.h"
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
!      logical :: pr_dry_depos
      integer :: lwi_flags (i1_gl:i2_gl, ju1_gl:j2_gl)
!      real*8  :: ai(k1-1:k2)
!      real*8  :: bi(k1-1:k2)
!      real*8  :: am(k1:k2)
!      real*8  :: bm(k1:k2)
      real*8  :: mcor  (i1:i2, ju1:j2)
      real*8  :: latdeg(ju1_gl:j2_gl)
      real*8  :: londeg (i1_gl:i2_gl)
!      real*8  :: humidity  (i1:i2, ju1:j2, k1:k2)
      real*8  :: max_cloud (i1:i2, ju1:j2, k1:k2)
      real*8  :: pbl       (i1:i2, ju1:j2)
      real*8  :: radswg    (i1:i2, ju1:j2)
      real*8  :: ran_cloud (i1:i2, ju1:j2, k1:k2)
      real*8  :: surf_air_temp(i1:i2, ju1:j2)
      real*8  :: surf_rough(i1:i2, ju1:j2)
      real*8  :: ustar     (i1:i2, ju1:j2)
croy      real*8  :: mass (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: mass      (i1:i2, ju1:j2, k1:k2)

      real*8  :: psf  (ilo:ihi, julo:jhi)
!      real*8  :: kel1 (ilo:ihi, julo:jhi, k1:k2)
croy      real*8  :: const(ilo:ihi, julo:jhi, k1:k2, num_species)
      real*8  :: const(i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: dry_depos(i1:i2, ju1:j2, 1:num_species)
      real*8  :: diffaer   (i1:i2,ju1:j2, 1:num_species)
      real*8  :: s_radius  (i1:i2,ju1:j2, 1:num_species)
      real*8  :: s_velocity(i1:i2,ju1:j2, 1:num_species)
c
c ADDED: Sillman 4/28/08.
c     Argument declarations from diffu_dao2.F

      real*8  :: kzz(i1:i2, ju1:j2, k1:k2)
      real*8  :: press3c(ilo:ihi, julo:jhi, k1:k2)
      real*8  :: press3e(ilo:ihi, julo:jhi, k1-1:k2)
! Guangxing Lin
       real*8  :: tdt
       real*8  :: box_zm     (i1:i2, ju1:j2)
       real*8  :: box_zi     (i1:i2, ju1:j2)
       real*8  :: drydep_vel(i1:i2, ju1:j2, 1:num_species)
       integer :: ireg   (i1:i2, ju1:j2)
       integer :: iland  (i1:i2, ju1:j2, 1:NTYPE)
       integer :: iuse   (i1:i2, ju1:j2, 1:NTYPE)
       real*8  :: xlai   (i1:i2, ju1:j2, 1:NTYPE) !leaf area index of
       integer :: chem_opt
!     land type elements



c     -----------------------
c     Parameter declarations.
c     -----------------------
 
      real*8, parameter :: GOLDER_A(6) =
     &  (/ -0.096d0, -0.037d0, -0.002d0, 1.0d-5,   0.004d0,  0.035d0 /)

      real*8, parameter :: GOLDER_B(6) =
     &  (/  0.029d0,  0.029d0,  0.018d0, 0.000d0, -0.018d0, -0.036d0 /)

c
c ADDED: Sillman 4/28/08.
c     Parameter declarations from diffu_dao2.F

      real*8, parameter :: HZERO  = 8000.0d0  ! atmospheric scale height (m)

      real*8, parameter :: HZERO2 = HZERO * HZERO  ! (m^2)



c     ----------------------
c     Function declarations.
c     ----------------------

      logical :: Is_Fixed_Const

 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      logical, save :: first = .true.

      integer :: ic, il, ij
      integer :: nsec_jan1

      integer :: pasquill(i1:i2)

      real*8  :: days
      real*8  :: decl
c ADDED: Sillman, 2008.  loc_depos made 2d
c     real*8  :: loc_depos(i1:i2)
      real*8  :: loc_depos(i1:i2, ju1:j2)
      real*8  :: rdistsq
      real*8  :: time

c ADDED: Sillman,4/28/08:  box_height is 2-d
c     real*8  :: box_height(i1:i2)
      real*8  :: box_height(i1:i2,ju1:j2)

      real*8  :: cloud_frac(i1:i2)
      real*8  :: csza      (i1:i2)
      real*8  :: cz1       (i1:i2)
      real*8  :: obk       (i1:i2)
      real*8  :: windspeed (i1:i2)

      real*8  :: dvel (i1:i2, num_species)
      real*8  :: diffa(i1:i2, num_species)
      real*8  :: s_ra (i1:i2, num_species)
      real*8  :: s_vel(i1:i2, num_species)
c
c ADDED: Sillman 4/28/08.
c     Variable  declarations from diffu_dao2.F

      integer :: ik
c     integer :: il, ij

      real*8  :: avdiff(i1:i2, ju1:j2, k1:k2-1)
      real*8  :: bvdiff(i1:i2, ju1:j2, k1:k2-1)
      real*8  :: cvdiff(i1:i2, ju1:j2, k1:k2-1)
      real*8  :: vdiff (i1:i2, ju1:j2, k1:k2-1)

      real*8  :: bb    (i1:i2, ju1:j2, k1:k2-1)
      real*8  :: cc    (i1:i2, ju1:j2, k1:k2-1)
      real*8  :: rr    (i1:i2, ju1:j2, k1:k2-1)

c ADDED: Sillman 4/28/08.
c   Additional variable declarations
      real*8  :: dvel2d (i1:i2, ju1:j2, num_species)    ! Deposition velocity
      real*8  :: depfr(i1:i2, ju1:j2, k1:k2-1)     !Dep. fraction for vert layer
      real*8  :: deprate(i1:i2, ju1:j2)          !Dimensionless removal rate
      real*8  :: sum (i1:i2, ju1:j2)               !normalization sum
      real*8  :: rrsum(i1:i2, ju1:j2)               !normalization sum
      real*8  ::  alpha(i1:i2, ju1:j2)               !normalization term

c   TEST WRITE variable declarations
!!      real*8 :: testdep (3, num_species+1)    ! masssum before, after, diff

c   Index for rmoh
!      integer, parameter :: icrmoh  =  120    !  *** enter index number here ***
!  Guangxing Lin got to change abvoe number 

c     ----------------
c     Begin execution.
c     ----------------

!      if (pr_diag) then
!        Write (6,*) 'Update_Drydep called by ', loc_proc
!      end if
c         Write (6,*) 'i1,i2,ju1,j2,k1,k2=', i1,i2,ju1,j2,k1,k2
c         Write (6,*) 'ilo, ihi,julo,jhi =',ilo,ihi,julo,jhi
c        if(k1.le.42.and.k2.ge.41) then
c  (il=67,ij=2,ik=1,2,3)
c         if(i1.le.67.and.i2.ge.67.and.ju1.le.1) then
          
c           do ik=1,15
c            write(6,*) 'i1,ju1,ik, const(i1,ju1,ik,icrmoh) = ',
c     &                i1,ju1,ik, const(i1,ju1,ik,icrmoh)
c            write(6,*) '67,02, ik, const(67, 02,ik,icrmoh) = ',
c     &                i1,ju1,ik, const(67,02,ik,icrmoh)
c           enddo
 
c        endif
        

c     ==========
      if (first) then
c     ==========

        first = .false.

!        if (pr_dry_depos) dry_depos(i1:i2,ju1:j2,1:num_species) = 0.0d0
         dry_depos(i1:i2,ju1:j2,1:num_species) = 0.0d0

      end if


c ADDED: Sillman 4/28/08
c  Initialize dvel2d=0 and testdep  (for debug).

             dvel2d(:,:,:) = 0.0d0
!!             testdep (:,:) = 0.0d0

c Begin dry deposition algorithm, ending with calculation of 
c                                               deposition velocity.

      pasquill(:) = 0

      csza(:)   = 0.0d0
      dvel(:,:) = 0.0d0


ccc!! For now just hardwire a few values for the radon/lead problem
c     (chem_opt = 1).  Setkin will eventually provide this information
c     for all species when doing a full chemistry calculation.

!      write(6,*) 'Roy->2'

       chem_opt =9 ! Guangxing Lin

      if (chem_opt == 1) then

        aerosol(1) = 0
        aerosol(2) = 1
  
        hstar(1)   = 9.3d-3
        hstar(2)   = 0.0d0
  
        oxidize(1) = 0.0d0
        oxidize(2) = 0.0d0

      else if (chem_opt == 6) then

        aerosol(1) = 12
        aerosol(2) = 12

        hstar(1)   = 0.0d0
        hstar(2)   = 0.0d0

        oxidize(1) = 0.0d0
        oxidize(2) = 0.0d0

      end if

c      Write (6,*) 'Update_Drydep after zeroes' 

      call Get_Sec_Jan1 (nsec_jan1, nymd, nhms)

      time = nsec_jan1
      days = time / SECPDY


c     ============
      call Solrpos
c     ============
     &  (days, decl, rdistsq)


c     ================
      do ij = ju1, j2
c     ================

c      if (pr_diag) then
c        Write (6,*) 'Update_Drydep start ij loop, ij=', ij
c      end if



c       -----------------------------------------------------------
c       Put reasonable values into met fields for testing purposes.
c       -----------------------------------------------------------
!     Guangxing Lin--
!     Instead of calculating inside, pass from outside
!        do il = i1, i2
!
!          cz1(il) =
!     &      29.3d0 * kel1(il,ij,k1) *
!     &      (1.0d0 + 0.61d-3*humidity(il,ij,k1)) *
!     &      Log (psf(il,ij) / (am(k1)*pt + bm(k1)*psf(il,ij)))

!          box_height(il,ij) =
!     &      29.3d0 * kel1(il,ij,k1) *
!     &      (1.0d0 + 0.61d-3*humidity(il,ij,k1)) *
!     &      Log (psf(il,ij) / (ai(k1)*pt + bi(k1)*psf(il,ij)))
!
!        end do
           cz1(:)= box_zm(:,ij)
           box_height(:, ij)=box_zi(:,ij)
c       -----------------------------------------------------
c       Calculate solar zenith angle at this latitude for all
c       longitudes.
c       -----------------------------------------------------

c       ===========
        call Solrza 
c       ===========
     &    (days, decl, latdeg(ij), londeg(i1), ilong, csza)


c       -------------------------------------------------------
c       Calculate the Monin-Obukhov length by first calculating
c       the Pasquill stability class (A-F = 1-6) and then use 
c       Golder's formula (see Seinfeld pp 509-511 1986).
c       -------------------------------------------------------

        windspeed(:) =
     &    (ustar(i1:i2,ij) / 0.4d0) *
     &    Log (10.0d0 / surf_rough(i1:i2,ij))

cc!?    Flint does not like the following equation; it thinks that 
cc      ran_cloud is unused.
        cloud_frac(:) =
     &    1.0d0 -
     &    ((1.0d0 - Maxval (max_cloud(i1:i2,ij,:), dim=2)) *
     &     (Product ((1.0d0 - ran_cloud(i1:i2,ij,:)), dim=2)))


        do il = i1, i2
 
          if (csza(il) < -0.05d0) then
            if (cloud_frac(il) > 0.4375001d0) then
              if      (windspeed(il) < 2.0d0) then
                pasquill(il) = 6
              else if (windspeed(il) < 3.0d0)  then
                pasquill(il) = 5
              else
                pasquill(il) = 4
              end if
            else
              if      (windspeed(il) < 3.0d0) then
                pasquill(il) = 6
              else if (windspeed(il) < 5.0d0) then
                pasquill(il) = 5
              else
                pasquill(il) = 4
              end if
            end if
          else 
            if (radswg(il,ij) > 700.0d0) then
              if      (windspeed(il) < 2.5d0) then
                pasquill(il) = 1
              else if (windspeed(il) < 5.0d0) then
                pasquill(il) = 2
              else
                pasquill(il) = 3
              end if
            else if (radswg(il,ij) < 350.0d0) then
              if      (windspeed(il) < 2.0d0) then
                pasquill(il) = 2
              else if (windspeed(il) < 5.0d0) then
                pasquill(il) = 3
              else
                pasquill(il) = 4
              end if
            else
              if      (windspeed(il) < 1.0d0) then
                pasquill(il) = 1
              else if (windspeed(il) < 4.0d0) then
                pasquill(il) = 2
              else if (windspeed(il) < 5.5d0) then
                pasquill(il) = 3
              else
                pasquill(il) = 4
              end if
            end if
          end if

        end do


        obk(:) =
     &    1.0d0 /
     &    (GOLDER_A(pasquill) +
     &     (GOLDER_B(pasquill) * Log (surf_rough(i1:i2,ij))))
 

c       --------------------------------------------------------
c       Calculate deposition velocities at this latitude for all
c       longitudes and all species.
c       --------------------------------------------------------

        do il = i1, i2
           do ic = 1, num_species
              diffa(il,ic) = diffaer(il,ij,ic)
              s_ra(il,ic)  = s_radius(il,ij,ic)
              s_vel(il,ic) = s_velocity(il,ij,ic)
           enddo
        enddo

c       ===========
        call Depvel
c       ===========
     &   (ilong, ij, radswg(:,ij), surf_air_temp(:,ij), csza, oxidize,
     &    hstar, mw, aerosol, ustar(:,ij), cz1, obk, cloud_frac,
     &     lwi_flags(i1,ij), dvel, ireg, iland, iuse, xlai,
     &    diffa, s_ra, s_vel,delh_298_over_r)

c       if (pr_diag) then
c        Write (6,*) 'Update_Drydep after Depvel   ij=', ij
c       end if


c ADDED: Sillman, 4/28/08
c  Save deposition velocity in 2-d array

          dvel2d(:,ij,:) = dvel(:,:)

c      if (pr_diag) then
c        Write (6,*) 'Update_Drydep after dvel2d:=  ij=', ij
c      end if

c
c ADDED: Sillman, 4/28/08
c   Original algorithm to update mixing ratios for dry deposition
c     is now used only if flag for vertical diffusion is off.
c
c   Otherwise it is replaced by the new algorithm
c    that includes vertical diffusion in calculating dry deposition (below).

c  added by Guangxing Lin
            diffu_opt=1  !input from namelist in IMPACT
            vert_diffu_coef=1.0d0  !input from namelist in IMPACT
                 if (diffu_opt .eq.  0) then
c
c   Subtract dry deposition from existing mixing ratio (const)
c
c       =======
        icloop: do ic = 1, num_species
c       =======
            advec_flag(ic) =1 !Guangxing Lin

          if ((advec_flag(ic) /= 1) .or. (Is_Fixed_Const(ic))) then
c           ============
            cycle icloop
c           ============
          end if

c         --------------------------------------------------------
c         Subtract mixing ratio amount from existing mixing ratio
c         of const.
c         --------------------------------------------------------

          loc_depos(i1:i2, ij)=
     &      const(i1:i2,ij,k1,ic) *
     &      (1.0d0 - Exp (-dvel(i1:i2,ic) 
     &                 * tdt / box_height(i1:i2, ij)))

          const(i1:i2,ij,k1,ic) =
     &      const(i1:i2,ij,k1,ic) - loc_depos(i1:i2, ij)

c         ------------------------------------------------------
c         Accumulate the total dry deposited in units of kg/m^2.
c         ------------------------------------------------------

!          if (pr_dry_depos) then

            dry_depos(i1:i2,ij,ic) =
     &        dry_depos(i1:i2,ij,ic) +
     &        loc_depos(i1:i2,ij) 
     &               * (mass(i1:i2,ij,k1) / mcor(i1:i2,ij)) *
     &        mw(ic) / MWTAIR

!          end if


        end do icloop

c ADD: Sillman, 4/28/08
c  End - control to update without vertical diffusion.

                 endif      !if (diffu_opt .eq.  0) then

        drydep_vel(:, ij, :) = dvel(:,:) ! Guangxing Lin


c  End do loop:  do ij = ju1, j2
c     ======
      end do
c     ======

c      if (pr_diag) then
c        Write (6,*) 'Update_Drydep after ij=ju1,j2 loop'    
c      end if



c =================================================
c ADDED: Sillman, 4/28/08
c
c Algorithm for dry deposition from several layers
c   using vertical diffusion calculation from diffu_dao2.F.  
c 
c Vertical diffusion calculation from diffu_dao2.F is inserted here
c  and modified to represent the diffusion of a species in the surface layer.
c  The resulting distribution is used to determine 
c     removal through dry deposition from each vertical layer.  
c
c The vertical diffusion algorithm (diffu_dao2.F) is:
c    X1 = [I-A]^-1  Xo  
c With dry deposition added
c   X2 = [I-A]^-1  (Xo - D) = X1 - [I-A]^-1 D
c           where D = dry deposition loss from layer 1.
c
c (Note: an exact combined solution would add drydep loss rate to 
c   the A matrix in the vertical diffusion calculation.)
c =================================================
c
c 
c Control for no vertical diffusion
c   If no vertical diffusion, use original dry deposition algorithm above.
c   Do only when k1=1    

                 if (diffu_opt /= 0.and.k1.eq.1) then

c     ----------------
c     Begin execution.  (diffu_dao2.F)
c     ----------------

      avdiff(:,:,:) = 0.0d0; bvdiff(:,:,:) = 0.0d0
      cvdiff(:,:,:) = 0.0d0; vdiff (:,:,:) = 0.0d0

      bb(:,:,:) = 0.0d0; cc(:,:,:) = 0.0d0
      rr(:,:,:) = 0.0d0


c       if (pr_diag) then
c        Write (6,*) 'Update_Drydep after zero in Sandy-add'
c       end if



c     --------------------------------------------------------
c     If there was no kzz in the met file, it was set to < 0.
c     In this case calculate a vertical diffusion coefficient,
c     otherwise use kzz.
c     --------------------------------------------------------

      if (Maxval (kzz) < 0.0d0) then

c       ------------------------------------------------------
c       Set up the vertical diffusion coefficients.
c       First put height (m) into vdiff, then put simple
c       parameterization for diffusion coefficient into vdiff,
c       then put it into units of per mb
c       ------------------------------------------------------

        vdiff = 0.0d0

        do ik = 1, 10

          vdiff(:,:,ik) =
     &      -HZERO *
     &      Log (press3e(i1:i2,ju1:j2,ik) / psf(i1:i2,ju1:j2))

          where (vdiff(:,:,ik) < pbl(:,:))

            vdiff(:,:,ik) =
     &        vert_diffu_coef *
     &          (1.0d0 -
     &           2.0d0 *
     &           (Abs (vdiff(:,:,ik) - (pbl(:,:) * 0.5d0)) / pbl(:,:)))

          elsewhere

            vdiff(:,:,ik) = 0.0d0

          end where

          vdiff(:,:,ik) =
     &      vdiff(:,:,ik) /
cc   &      HZERO2 * (press3e(i1:i2,ju1:j2,ik)**2) /
     &      HZERO2 *
     &      (press3e(i1:i2,ju1:j2,ik) * press3e(i1:i2,ju1:j2,ik)) /
     &      (press3c(i1:i2,ju1:j2,ik) - press3c(i1:i2,ju1:j2,ik+1))

        end do

      else

          vdiff(:,:,k1:k2-1) =
     &      kzz(:,:,k1:k2-1) /
cc   &      HZERO2 * (press3e(i1:i2,ju1:j2,k1:k2-1)**2) /
     &      HZERO2 *
     &      (press3e(i1:i2,ju1:j2,k1:k2-1) *
     &       press3e(i1:i2,ju1:j2,k1:k2-1)) /
     &      (press3c(i1:i2,ju1:j2,k1:k2-1) -
     &       press3c(i1:i2,ju1:j2,k1+1:k2))

      end if

c       if (pr_diag) then
c        Write (6,*) 'Update_Drydep after vdiff set'            
c       end if


c     -----------------------------------------------------------
c     Construct the tri-diagonal matrix which will be used in the
c     implicit vertical diffusion solution.
c       avdiff is the lower band
c       bvdiff is the main diagonal
c       cvdiff is the upper band
c     -----------------------------------------------------------

c     -----------------------
c     First do the top layer.
c     -----------------------

      avdiff(:,:,k2-1) = 0.0d0
      bvdiff(:,:,k2-1) =  vdiff(:,:,k2-2)
      cvdiff(:,:,k2-1) = -vdiff(:,:,k2-2)


c     ----------------------
c     Now do all the layers.
c     ----------------------

      avdiff(:,:,k1+1:k2-2) = -vdiff(:,:,k1+1:k2-2)
      bvdiff(:,:,k1+1:k2-2) = (vdiff(:,:,k1+1:k2-2) +
     &                         vdiff(:,:,k1  :k2-3))
      cvdiff(:,:,k1+1:k2-2) = -vdiff(:,:,k1  :k2-3)


c     ------------------------------------
c     Now do the layer next to the ground.
c     ------------------------------------

      avdiff(:,:,k1) = -vdiff(:,:,k1)
      bvdiff(:,:,k1) =  vdiff(:,:,k1)
      cvdiff(:,:,k1) = 0.0d0


c     ---------------------------------------------------
c     Now convert them to the proper dimensionless units.
c     ---------------------------------------------------

      avdiff(i1:i2,ju1:j2,k1:k2-1) =
     &  avdiff(i1:i2,ju1:j2,k1:k2-1) * tdt /
     &  (press3e(i1:i2,ju1:j2,k1-1:k2-2) -
     &   press3e(i1:i2,ju1:j2,k1:k2-1))

      bvdiff(i1:i2,ju1:j2,k1:k2-1) =
     &  bvdiff(i1:i2,ju1:j2,k1:k2-1) * tdt /
     &  (press3e(i1:i2,ju1:j2,k1-1:k2-2) -
     &   press3e(i1:i2,ju1:j2,k1:k2-1))  +
     &  1.0d0

      cvdiff(i1:i2,ju1:j2,k1:k2-1) =
     &  cvdiff(i1:i2,ju1:j2,k1:k2-1) * tdt  /
     &  (press3e(i1:i2,ju1:j2,k1-1:k2-2) -
     &   press3e(i1:i2,ju1:j2,k1:k2-1))

c     ---------------------
c     Now solve the system.
c     ---------------------

c       -----------------------------------------
c       Set up the right hand side of the system.
c       -----------------------------------------

        bb(:,:,k2-1) = bvdiff(:,:,k2-1)

c ADDED:  Sillman, 4/29/08
c  Originally, diffu_dao2.F had a loop to solve for all species
c   and set  rr(:,:,k2-1) = const(i1:i2,ju1:j2,k2-1,ic).
c  Here, rr = 1 for surface, 0 for all other layers

        rr(:,:,k2-1) = 0.

c      if (pr_diag) then
c        Write (6,*) 'Update_Drydep after rr set '
c      end if


c       ---------------------------------
c       Eliminate the lower diagonal (a).
c       ---------------------------------

        do ik = k2 - 1, k1 + 1, -1

          cc(:,:,ik) = cvdiff(:,:,ik) / bb(:,:,ik)
          rr(:,:,ik) =     rr(:,:,ik) / bb(:,:,ik)


          bb(:,:,ik-1) = bvdiff(:,:,ik-1) -
     &                   avdiff(:,:,ik-1) * cc(:,:,ik)

c ADDED: Sillman, 4/29/08
c    original: rr(:,:,ik-1) = const(i1:i2,ju1:j2,ik-1,ic) - ...
c    Here, const replaced with 1 for surface layer, 0 for all others.
c 
          rr(:,:,ik-1) = 0.   - 
     &                   avdiff(:,:,ik-1) * rr(:,:,ik)

          if(ik.eq.2) then
            rr(:,:,ik-1) = rr(:,:,ik-1) + 1.  
          endif 


        end do

c      if (pr_diag) then
c        Write (6,*) 'Update_Drydep Before solve-for-new-mixing'
c      end if


c       -------------------------------
c       Solve for the new mixing ratio.
c       -------------------------------

c ADDED: Sillman, 4/29/08
c  Original:  diffu_dao2.F solves for const().      
c  Here, rr() is used  as dummy variable (in place of const).
c  Instead of mixing ratio, this represents the fraction of dry deposition
c     from each layer.  
c
c rrsum = Mass sum, used for normalization below
c         and cons. mass test (should equal mass in layer 1)

        rr(:, :, k1)              = rr(:,:,k1) / bb(:,:,k1)

        do ik = k1 + 1, k2 - 1

          rr (:,:,ik)  =   rr(:,:,ik) -
     &      rr(:,:,ik-1)  * cc(:,:,ik)

        end do

        rrsum(:,:) = 0.0d0
        do ik=k1,k2-1
          rrsum(:,:)=rrsum(:,:)+rr(:,:,ik)
     &        * (mass(i1:i2,ju1:j2,ik) / mcor(i1:i2,ju1:j2)) 
c    &         * mw(ic) / MWTAIR - skipped for normalization sum
        enddo

c      if (pr_diag) then
c        Write (6,*) 'Update_Drydep after  solve-for-new-mixing'
c      end if

c
c TEST WRITE:  Conservation of mass

c        if(i1.le.67.and.i2.ge.67.and.ju1.le.02.and.j2.ge.02) then
c          testdep (1,num_species+1) = mass(67,02,k1)/mcor(67,02)
c          testdep (3,num_species+1) = testdep (1,num_species+1)
c     &       - rrsum(67,02)
c          write(6,*) ' DRY DEPOSITION CONSERVATION OF MASS TEST (67,02)'
c          write(6,*)  'mass-1, masssum, diff(0) =' , 
c     &            testdep (1,num_species+1), rrsum(67,02),
c     &             testdep (3,num_species+1)
c          write(6,*) 'rr(1-6) (should be decreasing=', 
c     &                                (rr(67,02,ik),ik=1,6)
c          write(6,*) "rr for ik=k1,k2-1.  k1,k2=", k1,k2
c          write(6,101) (rr(67,02,ik),ik=k1,k2-1)
c101       format(8(1pe10.3))
c          write(6,*) "dvel for all species"
c         write(6,101) (dvel2d(67,02,ic), ic=1,num_species)
c          write(6,101) (dvel2d(67,02,icrmoh) )
c        endif


c ADDED: Sillman, 4/29/08
c Loop to calculate dry deposition for all species 
c   based on original drydep algorithm and verrtical distribution
c   calculated from vertical mixing.
c
c     =======
      icloopv: do ic = 1, num_species
c     =======
!        write(6,*) "JIALEI-2",ic
        if ((advec_flag(ic) /= 1) .or. (Is_Fixed_Const(ic))) then
c         ============
          cycle icloopv
c         ============
        end if

c      if (pr_diag) then
c        Write (6,*) 'Update_Drydep: start icloopv  ic=  ' , ic
c      end if

c Establish dry deposition removal fraction from each layer.
c
c The exact algorithm for removal is
c   depos = deprate * depfr * surface mixing ratio
c      with depfr = rr from vertical diffusion algorithm above.
c
c This may result in excess removal if aloft mixing ratios
c   were reduced by advection/convection (which occur after vert. diffu.) 
c
c To prevent this, the deposition fraction is constrained 
c   so that removal rates from aloft  are lower than the surface removal rate.
c
c Basic algorithm for depfr without this protection:
c       depfr(i1:i2, ju1:j2,k1:k2-1) =  rr(i1:i2, ju1:j2,k1:k2-1)

        depfr(i1:i2, ju1:j2,k1) =  rr(i1:i2, ju1:j2,k1)

c        if (pr_diag) then
c        Write (6,*) 'After depfr, before sum, alpha, ic= ' , ic
c        end if


        sum(i1:i2,ju1:j2) = rr(i1:i2, ju1:j2,k1)
     &        * (mass(i1:i2,ju1:j2,k1) / mcor(i1:i2,ju1:j2)) 
        alpha(i1:i2,ju1:j2) = max(const(i1:i2, ju1:j2,k1,ic), 1.0d-30)

c Added diagnostic write Sandy 8-10-10

c        if (pr_diag) then
c        Write (6,*) 'After depfr, sum, alpha, before ik ' , ic

c        if(k1.le.41.and.k2.ge.41) then
c          write(6,*) 'i1,ju1, const(i1,ju1,41,146) = ',
c     &                i1,ju1, const(i1,ju1,41,146)
c        endif

c        end if

c ERROR OCCURS BELOW HERE
c il,ij,ik,alpha = 31 33 41  ic=146 LIMO (always 146?)
c   always with il=i1, ij=j1. 

croy        do ik=k1+1,k2-1

c         depfr(i1:i2, ju1:j2,ik) = min(  rr(i1:i2, ju1:j2,ik) , 
c    &          rr(i1:i2, ju1:j2,k1)
c    &      *   const(i1:i2, ju1:j2,ik,ic)/alpha(i1:i2, ju1:j2) )


croy          do il=i1,i2
croy            do  ij=ju1,j2
c              if(il.eq.i1.and.(ij.eq.ju1.or.ij.eq.33)
c     &                     .and.ik.eq.41.and.ic.eq.146) then
croy              if(ic.gt.145) then
croy                write(6,*) 'rr, const, alpha =',il,ij,ik,rr(il,ij,ik),
croy     &            const(il,ij,ik,ic),alpha(il,ij) 
croy              endif
croy              depfr(il,ij,ik) = min(rr(il,ij,ik),rr(il,ij,k1) 
croy     &           *const(il,ij,ik,ic)/alpha(il,ij))
croy              if(ic.gt.145) then
croy                write(6,*) 'depfr = ', depfr(il,ij,ik)
croy              endif

croy            enddo
croy          enddo
!          write(6,*) 'Roy->3'
croy          sum(i1:i2, ju1:j2)=sum(i1:i2, ju1:j2)+depfr(i1:i2, ju1:j2,ik)
croy     &        * (mass(i1:i2,ju1:j2,ik) / mcor(i1:i2,ju1:j2)) 
c    &          *  mw(ic) / MWTAIR - skipped for normalization sum

croy        end do       !do ik=k1,k2

        do ik=k1+1,k2-1
c        if(ic.eq.146) then
!          write(6,*) 'Roy->rr,const,alpha=',ic,ik
c        end if

c added 8-11-10: protection against erroneous negative const
          const(i1:i2,ju1:j2,ik,ic) = 
     $     max(const(i1:i2,ju1:j2,ik,ic),0.d0)

          depfr(i1:i2, ju1:j2,ik) = min(  rr(i1:i2, ju1:j2,ik) ,
     &          rr(i1:i2, ju1:j2,k1)
     &      *   const(i1:i2, ju1:j2,ik,ic)/alpha(i1:i2, ju1:j2) )
c added 8-11-10: protection against negative input const
c should be unnecessary if protection is added for const (just above)
c         depfr(i1:i2, ju1:j2,ik) = max( depfr(i1:i2, ju1:j2,ik),0.)

          sum(i1:i2, ju1:j2)=sum(i1:i2, ju1:j2)+depfr(i1:i2, ju1:j2,ik)
     &        * (mass(i1:i2,ju1:j2,ik) / mcor(i1:i2,ju1:j2))
c    &          *  mw(ic) / MWTAIR - skipped for normalization sum

c Added test write 8-10-10
c        if(i1.le.67.and.i2.ge.67.and.ju1.le.02.and.j2.ge.02) then
c          if(ic.eq.icrmoh) then
c               if(ik.eq.k1+1) then
c               write(6,*) 'TEST IN DEPFR LOOP: const(67,02,1, ',
c     &          'alpha =', const(67,02,1,icrmoh), alpha(67,02)
c             endif    
c             write(6,*) ' TEST IN DEPFR LOOP: ik=', ik
c             write (6,*) 'rr(ik), rr(1), alpha =',
c     &            rr(67,02,ik),rr(67,02,1), alpha(67,02)
c             write(6,*) 'Prior const(67,02,ik and 1) = ', 
c     &         ik, const(67,02,ik,icrmoh),const(67,02,1,icrmoh) 
c             write(6,*) 'mass, mcor, sum = ',
c     &        mass(67,02,ik), mcor(67,02), sum(67,02) 
c          endif
c        endif 
c end added test write

        end do       !do ik=k1,k2

!       write(6,*) 'Roy->3.1=',ic

        do ik=k1,k2-1
          depfr(i1:i2, ju1:j2,ik) = depfr(i1:i2, ju1:j2,ik)
     &          *rrsum(i1:i2, ju1:j2)/sum(i1:i2, ju1:j2)
        end do       !do ik=k1,k2 

!          write(6,*) 'Roy->4'

c TEST WRITE:  normalization adjustment  write

c         if(i1.le.67.and.i2.ge.67.and.ju1.le.02.and.j2.ge.02) then
c          if(ic.eq.icrmoh      ) then
c            write(6,*)' TEST ic,i, j,  rrsum,  adjusted sum:',
c     &                  ic, rrsum(67,02), sum(67,02)
c          endif
c        endif


c         if (pr_diag) then
c            Write (6,*) 'Update_Drydep: after normalization adj' ,ic
c         end if


c Establish dimensionless deposition rate from surface
c   (from original deposition rate algorithm)
c
c Note:  surface removal = const * (1-Exp(-dvel*depfr*tdt/boxheight)/depfr
c        removal rate reduces to dvel*tdt/boxheight when small.
c 
c Test:  deprate * const(layer 1) should equal total deposition by mass.
c
          deprate(i1:i2, ju1:j2) = 
     &      (1.0d0 - Exp (-dvel2d(i1:i2, ju1:j2, ic) 
     &                      *depfr(i1:i2, ju1:j2,k1)
     &                 * tdt / box_height(i1:i2, ju1:j2 )))
     &                 /depfr(i1:i2, ju1:j2,k1)

c TEST WRITE Sum for conservation of mass test.           
c        if(i1.le.67.and.i2.ge.67.and.ju1.le.02.and.j2.ge.02) then
c           testdep(1,ic) = 
c     &       const(67, 02, k1,ic)            
c     &          * deprate(67,02)         
c     &          * (mass( 67  , 02,  k1  ) / mcor(  67 ,  02  )) *
c     &          mw(ic) / MWTAIR
c        endif


c      if (pr_diag) then
c        Write (6,*) 'Update_Drydep after calc deprate      ' 
c      end if



c Loop to remove dry deposition from each layer
        do ik=k2-1, k1, -1

c Remove dry deposition from each layer
           loc_depos(i1:i2, ju1:j2) =
     &       const(i1:i2,ju1:j2,k1,ic)  
     &          * deprate(i1:i2, ju1:j2)
     &          * depfr(i1:i2,ju1:j2,ik)

           const(i1:i2,ju1:j2,ik,ic)  = 
     &       const(i1:i2,ju1:j2,ik,ic)  
     &        -  loc_depos(i1:i2, ju1:j2) 


c TEST WRITE: OUTPUT
c        if(i1.le.67.and.i2.ge.67.and.ju1.le.02.and.j2.ge.02) then
c          if(ic.eq.icrmoh) then
c           write(6,*) 'End of Drydep: loc_depos, deprate, depfr, const',
c     &      ' for (67,02,ik,rmoh)'
c           write(6,*) 'ik=', ik, loc_depos(67,02), deprate(67,02),
c     &      depfr(67,02,ik), const(67,02,ik,icrmoh)
c          endif
c        endif


c TEST WRITE  Sum for conservation of mass test
c        if(i1.le.67.and.i2.ge.67.and.ju1.le.02.and.j2.ge.02) then
c           testdep (2,ic) = testdep (2,ic)
c     &     +  loc_depos(67,02)
c     &        * (mass( 67  , 02,  ik  ) / mcor(  67 ,  02  )) *
c     &        mw(ic) / MWTAIR
c        endif

c         ------------------------------------------------------
c         Accumulate the total dry deposited in units of kg/m^2.
c         ------------------------------------------------------

!          if (pr_dry_depos) then

            dry_depos(i1:i2,ju1:j2,ic) = 
     &        dry_depos(i1:i2,ju1:j2,ic) + 
     &        loc_depos(i1:i2,ju1:j2) 
     &        * (mass(i1:i2,ju1:j2,ik) / mcor(i1:i2,ju1:j2)) *
     &        mw(ic) / MWTAIR

!          end if

        end do       !do ik=k2-1 , k1, -1

!          write(6,*) 'Roy->5 icloopv=',ic

c ADDED: Sillman, 4/28/08. 
c  TEST WRITE: Test for conservation of mass in dry deposition.  

c The deposited amount (testdep2)  should be equal to the deposition rate
c   applied to the initial concentration of layer 1 (testdep1).

c   dry dep is kg/m2  (mass=kg, mcor=m2)

c       if(i1.le.67.and.i2.ge.67.and.ju1.le.02.and.j2.ge.02) then
c         if(i1.le.67.and.i2.ge.67.and.ju1.le.02.and.j2.ge.02) then
c          if(ic.eq.icrmoh      ) then
c           testdep (3,ic) = testdep (2,ic) - testdep (1,ic)
c          if(ic.eq.1) write(6,*)'DRY DEPOSITION TEST: 67,02, k1=',k1,
c     &     'Test deprate = 1/fr(1-exp(dvel*fr*tdt/boxheight)'
c          if(ic.eq.1) write(6,*)
c     &     'Test dep1 = deprate*const1*mass = dep2' 
c          if(ic.eq.1) write(6,*)'tdt =', tdt,
c     &       'box_height=', box_height(67,02)
c     &      ,' avdiff = ',avdiff(67,02,k1)
c          if(ic.le.6) write(6,*)'ic,dvel,depfr, const, deprate=',
c     &          ic, dvel2d(67,02,ic), depfr(67, 02, 1), 
c
c     &              const(67,02,1,ic), deprate (67,02)
c         if(ic.le.9) write(6,*) 'ic, dep1, dep2, diff =',  ic,
c    &                testdep (1,ic), testdep (2,ic),testdep (3,ic)
c        endif
c       endif


c     =============
      end do icloopv
c     =============


c End vertical deposition algorithm from diffu_dao2.F
c
c End -Control to skip vertical diffusion
                 endif   ! if (diffu_opt /= 0) then
!          write(6,*) 'Roy->6'


!      if (pr_diag) then
!        Write (6,*) 'End Update_Drydep called by ', loc_proc
!      end if


      return
 
      end
 

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Depvel
c
c DESCRIPTION
c   This routine computes the dry deposition velocities using a
c   resistance-in-series model.
c
c   Routine reads data which:
c     - converts land type id to deposition surface type id
c     - gives roughness heights for each land type id
c     - identifies water land type id's, for stability and z0 calculations
c     - reads surface resistance data for each deposition surface type id
c
c   Changes from version 3.1 to version 3.2:
c     * In unstable atmospheres with |zlmo| < zo, as can happen occasionally
c       under very low wind conditions with tall canopies, application of
c       Monin-Obukhov similarity yields negative values for ra.  This was a
c       problem in version 3.1.  In fact, Monin-Obukhov similarity does not
c       apply under such conditions, so we now set ra to zero and let the
c       boundary resistance rb define the overall aerodynamic resistance.
c       Since rb varies inversely with u*, it will impose a large aerodynamic
c       resistance under very low wind conditions.
c     * The range of applicability of stability correction functions to
c       Monin-Obukhov similarity has been extended to -2.5 < z/zmo < 1.5,
c       based on Figure 2 of Businger et al. [1971].  The range used to be
c       -1 < z/zmo < 1 in version 3.1.
c                                  
c  Literature cited:
c     Baldocchi, D.D., B.B. Hicks, and P. Camara, A canopy stomatal
c       resistance model for gaseous deposition to vegetated surfaces,
c       Atmos. Environ. 21, 91-101, 1987.
c     Brutsaert, W., Evaporation into the Atmosphere, Reidel, 1982.
c     Businger, J.A., et al., Flux-profile relationships in the atmospheric
c       surface layer, J. Atmos. Sci., 28, 181-189, 1971.
c     Dwight, H.B., Tables of integrals and other mathematical data,
c       MacMillan, 1957.
c     Guenther, A., and 15 others, A global model of natural volatile
c       organic compound emissions, J. Geophys. Res., 100, 8873-8892, 1995.
c     Hicks, B.B., and P.S. Liss, Transfer of SO2 and other reactive
c       gases across the air-sea interface, Tellus, 28, 348-354, 1976.
c     Jacob, D.J., and S.C. Wofsy, Budgets of reactive nitrogen,
c       hydrocarbons, and ozone over the Amazon forest during the wet season,
c       J.  Geophys. Res., 95, 16737-16754, 1990.
c     Jacob, D.J., and 9 others, Deposition of ozone to tundra,
c       J. Geophys. Res., 97, 16473-16479, 1992.
c     Levine, I.N., Physical Chemistry, 3rd ed., McGraw-Hill, New York, 1988.
c     Munger, J.W., and 8 others, Atmospheric deposition of reactive
c       nitrogen oxides and ozone in a temperate deciduous forest and a
c       sub-arctic woodland, J. Geophys. Res., in press, 1996.
c     Walcek, C.J., R.A. Brost, J.S. Chang, and M.L. Wesely, SO2, sulfate,
c       and HNO3 deposition velocities computed using regional landuse and
c       meteorological data, Atmos. Environ., 20, 949-964, 1986.
c     Wang, Y.H., paper in preparation, 1996.
c     Wesely, M.L, Improved parameterizations for surface resistance to
c       gaseous dry deposition in regional-scale numerical models, 
c       Environmental Protection Agency Report EPA/600/3-88/025,
c       Research Triangle Park (NC), 1988.
c     Wesely, M.L., same title, Atmos. Environ., 23, 1293-1304, 1989.
c
c ARGUMENTS
c   npts    : tbd
c   ij      : tbd
c   radiat  : solar radiation (W*m^-2)
c   temp    : surface air temperature (degK)
c   suncos  : cosine of solar zenith angle
c   f0      : reactivity factor for oxidation of biological substances
c   hstar   : Henry's Law constant
c   xmw     : molecular weights; used to calculate molecular diffusivities
c             (kg/mole)
c   airosol : 0 => gas-phase species, 1 => aerosol species
c   ustar   : friction velocity (m/s)
c   cz1     : altitude at which deposition velocity is computed (m)
c   obk     : Monin-Obukhov length, set to 1.0d5 under neutral conditions (m)
c   cfrac   : fractional cloud cover
c   zh      : mixing depth (m)
c   lsnow   : integer for snow and sea ice (1=>water, 2=>land, 3=>ice)
c   dvel    : deposition velocities (m/s)
c   ireg    : # of landtypes in grid square
c   iland   : land type id for elements ldt = 1, ireg;
c             could be from any source, mapped to deposition surface id
c   iuse    : fraction of gridbox area occupied by land type elements (mil^-1)
c   xlai    : leaf area index of land type elements
c   diffa   : aerosol diffusivity (m^2/s)
c   s_ra    : aerosol radius (m)
c   s_vel   : aerosol settling velocity (m/s)
c
c-----------------------------------------------------------------------------

      subroutine Depvel
     &  (npts, ij, radiat, temp, suncos, f0, hstar, xmw, airosol,
     &   ustar, cz1, obk, cfrac, lsnow, dvel, ireg, iland,
     &   iuse, xlai, diffa, s_ra, s_vel, delh_298_over_r)

!  Guangxing Lin deleted zh, since it is not used in the code
!    added delh_298_over_r
       use h_imp_phys_constants
      use h_imp_dims, only: i1,i2, ju1,j2, num_species
      use h_imp_control, only: MAX_NUM_CONST, advec_flag
      use h_imp_emiss_constants
      implicit none

!#     include "imp_emiss_constants.h"
!#     include "imp_phys_constants.h"
!#     include "imp_control.h"
!#     include "imp_dims.h"
#     include "imp_drydep_data.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: npts
      integer :: ij
      real*8  :: radiat (npts)
      real*8  :: temp   (npts)
      real*8  :: suncos (npts)
      real*8  :: f0     (num_species)
      real*8  :: hstar  (num_species)
      real*8  :: xmw    (MAX_NUM_CONST)
      integer :: airosol(num_species)
      real*8  :: ustar  (npts)
      real*8  :: cz1    (npts)
      real*8  :: obk    (npts)
      real*8  :: cfrac  (npts)
c      real*8  :: zh     (npts)
      integer :: lsnow  (npts)
ccc   real*8  :: dvel   (npts, MAX_NUM_CONST)
      real*8  :: dvel   (npts, num_species)
      integer :: ireg   (i1:i2, ju1:j2)
      integer :: iland  (i1:i2, ju1:j2, 1:NTYPE)
      integer :: iuse   (i1:i2, ju1:j2, 1:NTYPE)
      real*8  :: xlai   (i1:i2, ju1:j2, 1:NTYPE)
      real*8  :: diffa  (npts, num_species)
      real*8  :: s_ra   (npts, num_species)
      real*8  :: s_vel  (npts, num_species)
      real*8  :: delh_298_over_r(num_species)!Guangxing Lin 
 
c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter :: XCKMAN = 0.4d0


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8  :: Diffg


c     ----------------------
c     Variable declarations.
c     ----------------------

      logical :: is_found

      logical :: ldep(MAX_NUM_CONST)

      logical :: lrgera(npts)  ! T -> stable atmosphere; a high aerodynamic
                               ! resistance (ra = 1.0d4 m/s) is imposed; else
                               ! ra is calculated

      integer :: idep1
      integer :: ijloop
      integer :: iolson
      integer :: iw
      integer :: k
      integer :: ldt
      integer :: nwaterx

      real*8  :: c1
      real*8  :: ckustr
      real*8  :: corr1
      real*8  :: cz       ! altitude where deposition velocity is computed (m)
      real*8  :: czh
      real*8  :: dair
      real*8  :: dummy1, dummy2, dummy3, dummy4
      real*8  :: ra, rb
      real*8  :: rdc
      real*8  :: reyno
      real*8  :: schno
      real*8  :: stono
      real*8  :: ebr, eim, ein, r1
      real*8  :: rix
      real*8  :: rt
      real*8  :: tempc    ! surface air temperatures in degC
      real*8  :: tempk    ! surface air temperatures in degK
      real*8  :: xnu
      real*8  :: z0obk

      real*8, save :: press = 1.5d5

      real*8  :: c1x (MAX_NUM_CONST)  ! total resistance to deposition for
                                      ! each species (s/m)
      real*8  :: vd  (MAX_NUM_CONST)
      real*8  :: vk  (MAX_NUM_CONST)              

      real*8  :: rac (NTYPE)
      real*8  :: rclo(NTYPE)
      real*8  :: rcls(NTYPE)
      real*8  :: rgso(NTYPE)
      real*8  :: rgss(NTYPE)
      real*8  :: ri  (NTYPE)
      real*8  :: rlu (NTYPE)

      real*8  :: zo  (NTYPE)  ! roughness height for specific surface types (m)

      real*8  :: rsurfc(MAX_NUM_CONST, NTYPE)  ! bulk surface resistance for
                                               ! each species (s/m)
       real*8  :: henry_eff(num_species) !Guangxing Lin

c     ----------------
c     Begin execution.
c     ----------------

c     --------------------------------------------------
c     If ldep(k)=F, species does not deposit.
c     Deposition is applied only to species with ldep=T.
c     --------------------------------------------------

      do k = 1, num_species
        ldep(k) =
     &    ((hstar(k) > 0.0d0) .or.
     &     (f0(k)    > 0.0d0) .or.
     &     (airosol(k) /=0))
      end do


      do k = 1, num_species 
        do ijloop = 1, npts
          dvel(ijloop,k) = 0.0d0
        end do
      end do


c     ============================
      IJLOOPX: do ijloop = 1, npts
c     ============================

c       ------------------------------
c       Compute deposition velocities.
c       ------------------------------

        cz = cz1(ijloop)

        tempk = temp(ijloop)
        tempc = temp(ijloop) + ABS_ZERO


        do k = 1, num_species
          vd(k) = 0.0d0
	  do ldt = 1, NTYPE
	    rsurfc(k,ldt) = 0.0d0
	  end do
        end do


c       ---------------------------------------------------------------------
c       Calculate the kinematic viscosity xnu (m^2/s) of air as a function of
c       temperature.  The kinematic viscosity is used to calculate the
c       roughness heights over water surfaces and to diagnose whether such
c       surfaces are aerodynamically rough or smooth using a Reynolds number
c       criterion.  The expression for the temperature dependence of xnu is
c       from the Fortran code in Appendix II of Wesely [1988].
c       ---------------------------------------------------------------------

	c1  = -tempk / ABS_ZERO

	xnu = 0.151d0 * (c1**1.77d0) * 1.0d-04


c       -----------------------------------------------------------------
c       Compute bulk surface resistance for gases.    
c
c       Adjust external surface resistances for temperature; 
c       from Wesely [1989], expression given in text on p. 1296.        
c
c       There is no evidence that the resistance continues to increase at
c       temperatures below -18 C, so at colder temperatures, hold the
c       resistance fixed.
c       -----------------------------------------------------------------

        rt = 1000.0d0 * Exp (-tempc - 4.0d0)

        if (tempc < -18.0d0) rt = 1.2d9


c       ------------------------------------------------------------------
c       Get surface resistances - loop over land types ldt.
c
c       The land types within each grid square are defined using the Olson
c       land type database.  Each of the Olson land types is assigned a 
c       corresponding "deposition land type" with characteristic values of
c       surface resistance components.  There are 74 Olson land-types, but
c       only 11 deposition land types (i.e., many of the Olson land types
c       share the same deposition characteristics).  Surface resistance
c       components for the "deposition land types" are from Wesely [1989],
c       except for tropical forests [Jacob and Wofsy, 1990] and for tundra
c       [Jacob et. al., 1992].  All surface resistance components are
c       normalized to a leaf area index of unity.
c 
c       Olson land types, deposition land types, and surface resistance
c       components are read from the dry deposition file; check that file
c       for further details.
c       ------------------------------------------------------------------

c       ==========================================
        LDTLOOP1: do ldt = 1, ireg(ijloop+i1-1,ij)
c       ==========================================

c                                            ==============
          if (iuse(ijloop+i1-1,ij,ldt) == 0) cycle LDTLOOP1
c                                            ==============

          if (lsnow(ijloop) == 3) then  ! snow or ice
            idep1  = 1
          else
            iolson = iland(ijloop+i1-1,ij,ldt) + 1
            idep1  = IDEP(iolson)
          end if


c         =======================
          call Surface_Resistance
c         =======================
     &      (idep1, rac(ldt), rclo(ldt), rcls(ldt), rgso(ldt),
     &       rgss(ldt), ri(ldt), rlu(ldt), rt, tempc,
     &       cfrac(ijloop), radiat(ijloop), suncos(ijloop),
     &       xlai(ijloop+i1-1,ij,ldt), rix)

c         ----------------------------------------------------------------
c         Compute aerodynamic resistance to lower elements in lower part
c         of the canopy or structure, assuming level terrain; equation (5)
c         of Wesely [1989].
c         ----------------------------------------------------------------

          rdc =
     &      100.0d0 *
     &      (1.0d0 + (1000.0d0 / (radiat(ijloop) + 10.0d0)))


c         =============================
          KLOOP1: do k = 1, num_species
c         =============================
            
            advec_flag(k)=1     !Guangxing Lin
            if (advec_flag(k) /= 1) then
c             ============
              cycle KLOOP1
c             ============
            end if

c           -----------------------------------------------------
c           Species-dependent corrections to resistances are from
c           equations (6)-(9) of Wesely [1989].
c           -----------------------------------------------------

            if (airosol(k) == 0) then

              if (ldep(k)) then
! added by Guangxing Lin, replace hstar(k) with henry_eff
               henry_eff(k) =
     &          hstar(k) *
     &          Exp (-delh_298_over_r(k) *
     &               ((1.0d0 / tempk) -
     &                (1.0d0 / 298.0d0)))

c               ======================
                call Canopy_Resistance
c               ======================
     &            (rdc, rix, press, tempk, f0(k), henry_eff(k), xmw(k),
     &             rac(ldt), rclo(ldt), rcls(ldt), rgso(ldt),
     &             rgss(ldt), rlu(ldt), rsurfc(k,ldt))

c               ----------------------------------------------------
c               Set min and max values for bulk surface resistances.
c               ----------------------------------------------------

                rsurfc(k,ldt) =
     &            Max (1.0d0, Min (rsurfc(k,ldt), 9999.0d0))

              end if

            else
c             -----------------------------------
c             Get surface resistance for aerosols   
c             eqs.(5)-(9) of Zhang et al. [2001]
c             -----------------------------------

              schno = xnu / diffa(ijloop,k)

              if (A_veg(idep1) .gt. 0.0d0) then
                stono = s_vel(ijloop,k) * ustar(ijloop) / 
     &                  (IMP_G * A_veg(idep1)) 

                ein = 2.0d0 * (s_ra(ijloop,k) / A_veg(idep1)) ** 2.0d0  

              else
                stono = s_vel(ijloop,k) * ustar(ijloop)**2.0d0 /
     &                  (IMP_G * xnu)
                ein = 0.0d0
              endif

              ebr = schno ** (-GAMA_Br(idep1))

              eim = (stono / (ALFA_im(idep1) + stono)) ** 2.0d0

              if (IWET(idep1) .gt. 0) then
                 r1 = 1.0d0
              else
                 r1 = exp(-sqrt(stono))
                 if (r1 .lt. 1.0d-10) r1 = 1.0d-10
              endif
  
              rsurfc(k,ldt) = 1.0d0 / (3.0d0 * ustar(ijloop) *
     &                        (ebr + eim + ein) * r1)

            end if

c         =============
          end do KLOOP1
c         =============

c       ===============
        end do LDTLOOP1
c       ===============


c       ==========================================
        LDTLOOP3: do ldt = 1, ireg(ijloop+i1-1,ij)
c       ==========================================

c         -------------------------------------------------------
c         Loop through the different landuse types present in the
c         grid square.
c         -------------------------------------------------------

c                                            ==============
          if (iuse(ijloop+i1-1,ij,ldt) == 0) cycle LDTLOOP3
c                                            ==============

          iolson = iland(ijloop+i1-1,ij,ldt) + 1


c         -------------------------------------------------------------
c         Get roughness heights; they are specified constants for each
c         surface type, except over water where zo = f(u*).  The latter
c         dependence is from equation (6) of Hicks and Liss [1976]. 
c         -------------------------------------------------------------

          nwaterx = NWATER

          is_found = .false.

          IWLOOP: do iw = 1, nwaterx

            if (iolson /= iwater(iw)) cycle IWLOOP

            zo(ldt) =
     &        1.4d-02 * ustar(ijloop) * ustar(ijloop) / 9.8d0 +
     &        1.1d-01 * xnu / ustar(ijloop)

            is_found = .true.

            exit IWLOOP

          end do IWLOOP

          if (.not. is_found) then
            zo(ldt) = izo(iolson)
            zo(ldt) = zo(ldt) * 1.0d-4
          end if


c         ------------------------------------------------------------------
c         Get aerodynamic resistances ra and rb.
c
c         The aerodynamic resistance ra is integrated from altitude z0+d up
c         to the altitude z1, at which the dry deposition velocity is to be
c         referenced.  The integration corrects for stability using
c         Monin-Obukhov similarity formulas from Businger et. al. [1971],
c         which apply over the range -2.5 < z/zMO < 1.5  (see their
c         Figure 2).  Under very unstable conditions when z1 > -2.5 zMO, we
c         assume that there is no resistance to transfer in the convective
c         column between zmo and z1.  Under very stable conditions when
c         z1 > 1.5 zMO, we assume that vertical transfer in the column
c         between zmo and z1 is strongly suppressed so that the deposition
c         velocity at altitude z1 is very low.  Under these conditions, we
c         just specify a very large ra=1.0d4 s/m (lrgera = T).
c
c         The Reynolds number reyno diagnoses whether a surface is
c         aerodynamically rough (reyno > 10) or smooth.
c         NOTE: The criterion "reyno > 10" is now replaced by "reyno > 1".

c         Surface is rough in all cases except over water with low wind
c         speeds.  In the smooth case, vertical transport IN THE SUBLAYER
c         near the surface is limited by molecular diffusion and is
c         therefore very slow; we assign a large value of ra + rb to
c         account for this effect. (djj, hyl, bmy, 5/8/00)
c
c         In the aerodynamically rough case, the expression for ra is as
c         given in equation (5) of Jacob et. al. [1992]:
c           ra = (1/ku*)*int(from z0 to z1) (phi(x)/z)dz
c         where x = (z-d)/zmo, z is the height above ground, and d is the
c         displacement height, which is typically 70-80% of the canopy
c         height [Brutsaert, 1982].  We change the vertical coordinate so
c         that z=0 at the displacement height; that's OK since for all
c         practical applications, z1 >> d.  In this manner, we don't need to
c         assume any specific value for the displacement height.  Applying
c         the variable transformation z -> x = z/zmo, the equation above
c         becomes:
c           ra = (1/ku*)*int(from x0 to x1) (phi(x)/x)dx   with x=z/zmo
c         Here phi is a stability correction function originally formulated
c         by Businger et. al. [1971] and given in eqns 5a and 5b of Jacob
c         et. al. [1992].
c         For unstable conditions:
c           phi(x) = a/sqrt(1-bx)  where a=0.74, b = 9
c         The analytical solution to the integral is [Dwight, 1957, integral
c         192.11]:
c           int(dx/(x*sqrt(1-bx))) = log(abs((sqrt(1-bx)-1)/(sqrt(1-bx)+1)))
c         which yields the expression for ra used in the code for unstable
c         conditions.
c         For stable conditions,
c           phi(x) = a + bx        where a=0.74, b = 4.7
c         and the analytical solution to the integral is:
c           int((a/x)+b)dx = a*ln(x) + bx
c         which yields the expression of ra used in the code for stable
c         conditions.
c
c         The formulation of rb for gases is equation (12) of Walcek et. al.
c         [1986].  
c         ------------------------------------------------------------------

          ckustr = XCKMAN * ustar(ijloop)

          reyno = ustar(ijloop) * zo(ldt) / xnu 

          if (obk(ijloop) == 0.0d0) then

            Write (6,910) obk(ijloop), ijloop, ldt                

 910        format (1x, 'obk(ijloop) = ', e11.2, 1x, 'ijloop = ',
     &              i4, 1x, 'ldt = ', i3, /) 

          end if

          corr1 = cz / obk(ijloop)

          z0obk = zo(ldt) / obk(ijloop)

          lrgera(ijloop) = .false.

          if (corr1 > 0.0d0) then
            if (corr1 >   1.5d0) lrgera(ijloop) = .true.
          else if (corr1 <= 0.0d0) then
            if (corr1 <= -2.5d0) corr1 = -2.5d0
          end if

          if (ckustr == 0.0d0) then

            Write (6,920) ijloop, ckustr, XCKMAN, ustar(ijloop)

 920        format (1x, 'ijloop = ', i4, 1x, 'ckustr = ', e10.1, 1x,
     &              'XCKMAN = ', e12.4, 1x, 'ustar(ijloop) = ', e12.4)

cc?
            Close (98)

!            call Stopcode (" ")  ! debug
            
              stop "ckustr == 0.0d0 in  Depvel"

          end if


c         ------------------------------------------------------------------
c         Aerodynamically rough or smooth surface.
c
c         In the classic study by Nikuradse (1933) the transition from
c         smooth to rough was examined in pipe flow.  He introduced a
c         roughness Reynolds number rr = u* z0 / nu, and found the flow to
c         be smooth for rr < 0.13, and rough for rr > 2.5, with a transition
c         regime in between (E.B. Kraus and J.A. Businger, Atmosphere-Ocean
c         Interaction, second edition, p.144-145, 1994).  Similar statements
c         can be found in the books:
c           Evaporation into the Atmosphere, by Wilfried Brutsaert, p.59,89,
c             1982; or
c           Seinfeld & Pandis, p.858, 1998.
c         Here we assume a sudden transition point rr = 1 from smooth to
c         rough, following L. Merlivat (1978, The dependence of bulk
c         evaporation coefficients on air-water interfacial conditions as
c         determined by the isotopic method, J. Geophys. Res.,
c         Oceans & Atmos., 83, C6, 2977-2980).  Also refer to Brutsaert's
c         book, p.125.  we used to use the criterion "reyno > 10" for
c         aerodynamically rough surface and now change to "reyno > 1".
c         (hyl, 10/15/99)
c         ------------------------------------------------------------------

c         ===================
          if (reyno >= 1.0d0) then
c         ===================

c           ------------------------------
c           Aerodynamically rough surface.
c           ------------------------------

            if ((corr1 <= 0.0d0) .and. (z0obk < -1.0d0)) then

c             ----------------------------------------
c             Unstable condition with Abs (zlmo) < Z0;
c             set ra to zero (version 3.2).
c             ----------------------------------------

              ra = 0.0d0

            else if ((corr1 <= 0.0d0) .and. (z0obk >= -1.0d0)) then

c             ---------------------------------------------------
c             Unstable conditions; compute ra as described above.
c             ---------------------------------------------------

              dummy1 = (1.0d0 - (9.0d0 * corr1))**0.5d0
              dummy2 = (1.0d0 - (9.0d0 * z0obk))**0.5d0
              dummy3 = Abs ((dummy1 - 1.0d0) / (dummy1 + 1.0d0))
              dummy4 = Abs ((dummy2 - 1.0d0) / (dummy2 + 1.0d0))

              ra = 0.74d0* (1.0d0 / ckustr) * Log (dummy3 / dummy4)
                                 
            else if ((corr1 > 0.0d0) .and.
     &               (.not. lrgera(ijloop))) then

c             -----------------------------------------
c             Moderately stable conditions (z/zmo < 1);
c             compute ra as described above
c             -----------------------------------------

              ra =
     &          (1.0d0 / ckustr) *
     &          (0.74d0 * Log (corr1 / z0obk) +
     &           4.7d0  * (corr1 - z0obk))

            else if (lrgera(ijloop)) then

c             -----------------------
c             Very stable conditions.
c             -----------------------

              ra = 1.0d+04

            end if


            if (ra < 0.0d0) then

c             ------------------------------------------------------------
c             If ra is negative (as occasionally happened in version 3.1),
c             send a warning message.
c             ------------------------------------------------------------

              Write (6,930) ra, cz, zo(ldt), obk(ijloop)

 930          format ('WARNING:  ra < 0 in Depvel', 4(1x, e12.5))

            end if


            KLOOP3: do k = 1, num_species

c             ------------------------------------
c             Get total resistance for deposition.
c             ------------------------------------

              if ((advec_flag(k) /= 1) .or. (.not. ldep(k))) then
c               ============
                cycle KLOOP3
c               ============
              end if

              if (airosol(k) == 0) then 

c               ------------------------------------------------------------
c               dair is the thermal diffusivity of air; value of
c               0.2*1.E-4 m^2/s, cited on p. 16,476 of Jacob et. al. [1992].
c               ------------------------------------------------------------

                dair = 0.2d0 * 1.0d-4

                rb = (2.0d0 / ckustr) *
     &            (dair / Diffg (tempk, press, xmw(k)*KGPG))**0.667d0

                c1x(k) = ra + rb + rsurfc(k,ldt)

              else

                c1x(k) = ra + rsurfc(k,ldt)

              end if

            end do KLOOP3

c         ====
          else
c         ====

c           -------------------------------
c           Aerodynamically smooth surface.
c           -------------------------------

            KLOOP4: do k = 1, num_species

              if ((advec_flag(k) /= 1) .or. (.not. ldep(k))) then
c               ============
                cycle KLOOP4
c               ============
              end if

c             --------------------------------------------------------------
c             Suppress drydep over smooth surfaces by setting ra to a 
c             large value.  This prevents negative dry deposition 
c             velocities when ustar is very small (djj, bmy, 5/8/00) 
c             --------------------------------------------------------------

              ra = 1.0d4

              c1x(k) = ra + rsurfc(k,ldt)

            end do KLOOP4

c         ======
          end if
c         ======


c         ----------------------------------------------------------------
c         iuse is the fraction of the grid square occupied by surface ldt
c         in units of mil^-1 (iuse=500 => 50% of the grid square).  Add
c         the contribution of surface type ldt to the deposition velocity;
c         this is a loop over all surface types in the gridbox.
c         ----------------------------------------------------------------

          KLOOP5: do k = 1, num_species

            if ((advec_flag(k) /= 1) .or. (.not. ldep(k))) then
c             ============
              cycle KLOOP5
c             ============
            end if

            vk(k) = vd(k)
            vd(k) = iuse(ijloop+i1-1,ij,ldt)

            if (airosol(k) == 0) then 
              vd(k) = vk(k) + (0.001d0 * vd(k) / c1x(k))
            else
              vd(k) = vk(k) + (0.001d0 * vd(k) * (1./c1x(k) + 
     &                         s_vel(ijloop,k)))
            end if

          end do KLOOP5


c       ===============
        end do LDTLOOP3
c       ===============


        KLOOP6: do k = 1, num_species

          if ((advec_flag(k) /= 1) .or. (.not. ldep(k))) then
c           ============
            cycle KLOOP6
c           ============
          end if

          dvel(ijloop,k) = vd(k)

        end do KLOOP6


c     ==============
      end do IJLOOPX
c     ==============


      return                       

      end                          


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Canopy_Resistance
c
c DESCRIPTION
c   This routine calculates bulk surface resistance of the canopy from the
c   network of resistances in parallel and in series.
c
c ARGUMENTS
c   rdc     : tbd
c   rix     : tbd
c   press   : tbd
c   tempk   : tbd
c   f01     : tbd
c   hstar1  : tbd
c   xmw1    : tbd
c   rac1    : tbd
c   rclo1   : tbd
c   rcls1   : tbd
c   rgso1   : tbd
c   rgss1   : tbd
c   rlu1    : tbd
c   rsurfc1 : tbd
c
c-----------------------------------------------------------------------------

      subroutine Canopy_Resistance
     &  (rdc, rix, press, tempk, f01, hstar1, xmw1,
     &   rac1, rclo1, rcls1, rgso1, rgss1, rlu1, rsurfc1)
    
      use h_imp_phys_constants
      implicit none

!#     include "imp_phys_constants.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8  :: rdc
      real*8  :: rix
      real*8  :: press
      real*8  :: tempk
      real*8  :: f01
      real*8  :: hstar1
      real*8  :: xmw1
      real*8  :: rac1
      real*8  :: rclo1
      real*8  :: rcls1
      real*8  :: rgso1
      real*8  :: rgss1
      real*8  :: rlu1
      real*8  :: rsurfc1


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter ::
     &  BIG1 = 1.0d22


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8  :: Diffg


c     ----------------------
c     Variable declarations.
c     ----------------------

      real*8  :: dtmp1, dtmp2, dtmp3, dtmp4

      real*8  :: rclx
      real*8  :: rgsx
      real*8  :: rixx
      real*8  :: rluxx


c     ----------------
c     Begin execution.
c     ----------------

      rixx =
     &  rix *
     &  Diffg (tempk, press, MWTH2O*KGPG) /
     &  Diffg (tempk, press, xmw1  *KGPG) +
     &  1.0d0 /
     &  (hstar1 / 3000.0d0 + 100.0d0 * f01)


      if (rlu1 < 9999.0d0) then
        rluxx = rlu1 / (hstar1 / 1.0d+05 + f01)
      else
        rluxx = BIG1
      end if

c     ---------------------------------------------------------------------
c     To prevent virtually zero resistance to species with huge HSTAR, such
c     as HNO3, a minimum value of rluxx needs to be set.  The rationality
c     of the existence of such a minimum is demonstrated by the observed
c     relationship between Vd(NOy-NOx) and ustar in Munger et. al. [1996];
c     Vd(HNO3) never exceeds 2 cm*s^-1 in observations.  The corresponding
c     minimum resistance is 50 s*m^-1.  This correction was introduced by
c     J.Y. Liang on 7/9/95.
c     ---------------------------------------------------------------------

      rluxx = Max (rluxx, 50.0d0)


      rgsx =
     &  1.0d0 /
     &  (hstar1 / 1.0d+05 / rgss1 + f01 / rgso1)

      rclx =
     &  1.0d0 /
     &  (hstar1 / 1.0d+05 / rcls1 + f01 / rclo1)


c     -----------------------------------------------------------------------
c     Get the bulk surface resistance of the canopy, rsurfc, from the network
c     of resistances in parallel and in series (Fig. 1 of Wesely [1989]).
c     -----------------------------------------------------------------------

      dtmp1 = 1.0d0 / rixx
      dtmp2 = 1.0d0 / rluxx
      dtmp3 = 1.0d0 / (rac1 + rgsx)
      dtmp4 = 1.0d0 / (rdc + rclx)

      rsurfc1 = 1.0d0 / (dtmp1 + dtmp2 + dtmp3 + dtmp4)


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Surface_Resistace
c
c DESCRIPTION
c   This routine tbd
c
c ARGUMENTS
c   idep1   : tbd
c   rac1    : tbd
c   rclo1   : tbd
c   rcls1   : tbd
c   rgso1   : tbd
c   rgss1   : tbd
c   ri1     : tbd
c   rlu1    : tbd
c   rt      : tbd
c   tempc   : tbd
c   cfrac1  : tbd
c   radiat1 : tbd
c   suncos1 : tbd
c   xlai1   : tbd
c   rix     : tbd
c
c-----------------------------------------------------------------------------

      subroutine Surface_Resistance
     &  (idep1, rac1, rclo1, rcls1, rgso1, rgss1, ri1, rlu1,
     &   rt, tempc, cfrac1, radiat1, suncos1, xlai1, rix)
        
       use h_imp_emiss_constants 
      implicit none

c#     include "imp_emiss_constants.h"
#     include "imp_drydep_data.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: idep1
      real*8  :: rac1
      real*8  :: rclo1
      real*8  :: rcls1
      real*8  :: rgso1
      real*8  :: rgss1
      real*8  :: ri1
      real*8  :: rlu1
      real*8  :: rt
      real*8  :: tempc
      real*8  :: cfrac1
      real*8  :: radiat1
      real*8  :: suncos1
      real*8  :: xlai1
      real*8  :: rix


c     -----------------------
c     Parameter declarations.
c     -----------------------

      real*8, parameter ::
     &  BIG1 = 1.0d6

      real*8, parameter ::
     &  BIG2 = 1.0d22


c     ----------------------
c     Function declarations.
c     ----------------------

      real*8  :: Biofit


c     ----------------------
c     Variable declarations.
c     ----------------------

      real*8  :: gfaci, gfact

      real*8  :: xdrycoeff(NPOLY)


c     ----------------
c     Begin execution.
c     ----------------

c     ----------------------------------------------------------------------
c     Read the internal resistance ri (minimum stomatal resistance for water
c     vapor, per unit area of leaf) from the IRI array; a "9999" value means
c     no deposition to stomata, so we impose a very large value for ri.
c     ----------------------------------------------------------------------

      ri1 = iri(idep1)

      if (ri1 >= 9999.0d0) ri1 = 1.0d22


c     ----------------------------------------------------------------------
c     Cuticular resistances IRLU read in from drydep table are per unit area
c     of leaf; divide them by the leaf area index to get a cuticular
c     resistance for the bulk canopy.  If IRLU is "9999", it means there are
c     no cuticular surfaces on which to deposit, so we impose a very large
c     value for rlu.
c     ----------------------------------------------------------------------

      if ((irlu(idep1) >= 9999) .or. (xlai1 <= 0.0d0)) then

        rlu1 = 1.0d6

      else

        rlu1 = irlu(idep1)
        rlu1 = (rlu1 / xlai1) + rt

      end if


c     ----------------------------------------------------------
c     The following are the remaining resistances for the Wesely
c     resistance-in-series model for a surface canopy
c     (see Atmos. Environ. paper, Fig.1).  
c     ----------------------------------------------------------

      rac1  = Max (irac(idep1), 1)
      if (rac1  >= 9999.0d0) rac1  = BIG2

      rgss1 = irgss(idep1)
      rgss1 = Max (rgss1+rt, 1.0d0)

      rgso1 = irgso(idep1)
      rgso1 = Max (rgso1+rt, 1.0d0)
      if (rgso1 >= 9999.0d0) rgso1 = BIG2

      rcls1 = ircls(idep1)
      rcls1 = rcls1 + rt
      if (rcls1 >= 9999.0d0) rcls1 = BIG2

      rclo1 = irclo(idep1)
      rclo1 = rclo1 + rt
      if (rclo1 >= 9999.0d0) rclo1 = BIG2


c     ----------------------------------------------------------------------
c     Adjust stomatal resistances for insolation and temperature =>
c
c     Temperature adjustment is from Wesely [1989], equation (3).
c       
c     Light adjustment by the function Biofit is described by Wang [1996].
c     It combines:
c       - local dependence of stomal resistance on the intensity I of light 
c         impinging the leaf; this is expressed as a mutliplicative 
c         factor I/(I+b) to the stomatal resistance where b = 50 W*m^-2
c         (equation (7) of Baldocchi et. al. [1987]);
c       - radiative transfer of direct and diffuse radiation in the 
c         canopy using equations (12)-(16) from Guenther et. al. [1995];
c       - separate accounting of sunlit and shaded leaves using
c         equation (12) of Guenther et. al. [1995];
c       - partitioning of the radiation at the top of the canopy into direct
c         and diffuse components using a parameterization to results from
c         an atmospheric radiative transfer model [Wang, 1996].
c     The dependent variables of the function Biofit are the leaf area
c     index (xylai), the cosine of zenith angle (suncos) and the fractional
c     cloud cover (cfrac).  The factor gfaci integrates the light
c     dependence over the canopy depth; sp even though ri is input per
c     unit area of leaf, it need not be scaled by lai to yield a bulk
c     canopy value because that is already done in the gfaci formulation.
c     ----------------------------------------------------------------------

      if (ri1 >= 9999.0d0) then

        rix = ri1

      else

        if ((tempc > 0.0d0) .and. (tempc < 40.0d0)) then
          gfact = 400.0d0 / tempc / (40.0d0 - tempc)
        else
          gfact = 100.0d0
        end if

        if ((radiat1 > 1.0d-5) .and. (xlai1 > 0.0d0)) then
          xdrycoeff(:) = DRYCOEFF(:)
          gfaci = 1.0d0 /  Biofit (cfrac1, suncos1, xlai1, xdrycoeff)
        else
          gfaci = 100.0d0
        end if

        rix = ri1 * gfact * gfaci

      end if


      return

      end

