c=============================================================================
c
c $Id: chemwrite.F,v 1.1 2003/05/16 22:22:25 akinorii Exp $
c
c CODE DEVELOPER
c   Sandy Sillman, U of M. (2007 version)
c
c FILE
c   chemwrite.F
c
c ORIGINAL FILE
c   rbcheminit.f
c
c=============================================================================
c
c ROUTINES:  chemwrite, analyze.  
c    chemwrite.f: Write complete output for chemistry
c    analyze.f:   Writes output relating to specif chemical species

c  cheminit.f   April, 2007
c
c  Program  written by Sanford Sillman
c  ---------------------------------------------------------
c  Program history:
c    12/06 Initial program by Sandy Sillman based on boxchemv7.f 
c  ---------------------------------------------------------
c     
c
       subroutine chemwrite( kw,
     &    c_xcin,  c_xcout,
     &    xcfinr,  rateh, rhdif, ratek, rateq, rateqq,
     &       c_rr, c_rp, c_rl
c    &     ,xrr, rpro, rloss 
     &     , c_h2oliq, c_temp, c_iter 
     &   )

c

c This writes chemical concentrations and reaction rates (gas+aqueous)
c   for the specified vector loop (kw).
c
c Written to the output file (lout)
c 
c Called by:  boxmain (as part of output at specified time steps)
c Calls to:   None.
c
c ---------------------------------------------
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c  7/07  Modified for impact
c
c -------------------------------------------------------------------



c -------------------------------------------------------------------
       use h_rbchemmech
       use h_rbchemvars, only: c_hour,c_idate, c_lat, c_lon, c_jparam
       use h_rbchemlocal, only: kk,ncf, nc, nc1, nc2,nn, avogadrl,nrh,
     & ic, i,j,neq, alpha, beta, gamma, nr, nrq,nrqq  
       IMPLICIT NONE

c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c#     include "rbchemlocal.h"
c
c VARIABLES FROM rbchemarrays, rbchemlocarr
      real*8 ::  c_xcin( c_kvec,c_cdim) ! concentr molec/cm3
      real*8 ::  c_xcout( c_kvec,c_cdim) ! concentr, mol/cm3

      real*8 :: xcfinr( c_kvec,c_cdim) ! ratio FINAL/AVG cncn
      real*8 :: ratek(c_kvec,c_rdim)  ! Rate constants
      real*8 :: rateh(c_kvec,c_hdim)  ! Henry's law constants
      real*8 :: rateq(c_kvec,c_aqdim)  ! Aqueous eq. constants
      real*8 :: rateqq(c_kvec,c_aqqdim) ! Special eq. constants
      real*8 :: rhdif(c_kvec,c_hdim) ! H. mod by diffusion

      real*8 ::  c_rp(c_kvec,c_cdim)  ! Production, molec/cm3
      real*8 ::  c_rl(c_kvec,c_cdim)  ! Loss, molec/cm3
      real*8 ::  c_rr(c_kvec,c_rdim)  ! Reaction rate m/cm3

        integer :: c_iter                  ! chem. number of iterations
         real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
         real*8 ::  c_temp(c_kvec)   ! temperature 

       integer :: kw
c LOCAL VARIABLES 

      real*8 :: xrr(c_kvec,c_cdim)    ! Solver concentration
      real*8 :: rloss(c_kvec,c_cdim)  ! Solver loss rate
      real*8 :: rpro(c_kvec,c_cdim)   ! Solver production rate

      character*8 :: tsum          ! 'SUM' name for output
      real*8 :: xcgas    ! Gas phase concentration
      real*8 :: acquacon ! Aqueous   concentration

c -------------------------------------------------------------------
        if( kw.le.0) return
        kk=1
c        write(6,*) 'chemwrite called'

c  SPECIES CONCENTRATIONS.
       write(c_out,1141)  c_hour
1141   format(/,' SPECIES CONCENTRATIONS AT  XHOUR =',  f11.2)
       ncf = int(0.2*(float(c_nchem2)-0.01))+1
       do      nc=1,ncf
         nc1 = 4*(nc-1) + 1
          nc2 = nc1 + 3
          if(nc1.gt.c_nchem2)nc1=c_nchem2
          if(nc2.gt.c_nchem2)nc2=c_nchem2
           write(c_out,1142) 
     *        ( c_tchem(nn),c_xcout(kw ,nn) , nn=nc1,nc2 )
1142     format(4(a8,1pe10.3,2x ))
       enddo      

c   AQUEOUS SPECIES CONCENTRATIONS

c NOTE  c_xcout(kw,ic) for GAS SPECIES IS GAS+AQ SUM, molec/cm3 equiv.
c       c_xcout(kw,icq) for AQUEOUS SPECIES is moles/liter
c    .  GAS-ONLY IS CALCULATED HERE.

c NOTE!  WRITE error for lumped species such as  MP (in rooh);
c        c_xcout(kw,ic) is preserved as species fraction, not sum?

          if(c_h2oliq(kw ).gt.0) then
           acquacon = c_h2oliq( kw)*avogadrl
           write(c_out,1301)   c_h2oliq(kw )
           write(c_out,1303) acquacon
           if(c_aqueous(1,2).gt.0) then
           write(c_out,1302) c_xcout(kw ,c_aqueous(1,2))
          endif

1301        format(/,'AQUEOUS CHEMISTRY ',/,'WATER (grams/cm3) =',
     *    1pe10.3)
1302        format( ' [H+] (moles per liter) =',1pe10.3)
1303        format( ' GAS SPECIES and G-A SUM molec/cm3. ',
     *      ' AQUEOUS moles/liter. CONVERSION= ', 1pe10.3)

c 1303        format( ' GAS SPECIES is G+A sum, molec/cm3. ',
c    *      ' AQUEOUS moles/liter. CONVERSION= ', 1pe10.3)

            tsum = '     SUM'
c          do      ic=1,c_nchem2
           do  nrh=1,c_nreach
             ic = c_henry(nrh,1)
             if(c_nequil(ic).gt.0)  then
               xcgas = c_xcout(kw,ic)
             do i=1,c_nequil(ic)
               xcgas = xcgas - c_xcout(kw,c_ncequil(ic,i))*acquacon
             enddo
c             write(c_out,1305) c_tchem(ic), c_xcout(kw ,ic),
c    *        ( c_tchem(c_ncequil(ic,i)),c_xcout(kw ,c_ncequil(ic,i))
c    *                                      ,i=1,c_nequil(ic) )
               write(c_out,1305) c_tchem(ic), xcgas,
     *        ( c_tchem(c_ncequil(ic,i)),c_xcout(kw ,c_ncequil(ic,i))
     *                                      ,i=1,c_nequil(ic) )
     *        ,tsum, c_xcout(kw,ic)
1305           format(4(a8,1pe10.3,2x),/,
     *          '         0.000E+00  ',3(a8,1pe10.3,2x))
               if(rhdif(kw,nrh).gt.0) 
     *         xcgas = (c_xcout(kw,c_ncequil(ic,1))*acquacon)
     *         /rhdif(kw,nrh)
              write(c_out,1306)xcgas, rateh(kw ,nrh), rhdif(kw ,nrh)
1306         format('   (',1pe10.3,
     *       ')                      HENRY, Hw/DIFF=', 3(1pe10.3))
c             write(c_out,1306) rateh(kw ,nrh), rhdif(kw ,nrh)
c 1306         format('                                ',
c    *       '     HENRY, Hw/DIFF=',2(1pe10.3))
            endif
           enddo    
         endif


c --------------
c CHEMWRIT  SUMMARY WRITE: 
c    CONCENTRATIONS, XCIN, RP, RL from QUADCHEM
c --------------

        write(c_out,1801) c_iter, kw 
1801    format(//,' SUMMARY WRITE:  ITER =',i3, '    VECTOR KKW=',i3)
       write(c_out,1804) c_IDATE, c_hour,  c_lat(1), c_lon(1),c_temp(1)
1804   format('IDATE xhour lat lon temp=',i8,4f8.2)
       write(c_out,1805) (c_jparam(i),i=1,13),c_jparam(20)
1805   format('JPARAMS: zenith altmid dobson so2 no2 aaerx aerssa',
     *  ' albedo',/,'cld-below cld-above claltB claltA temp date',
     *   /,(8f10.3))
        write(c_out,1802)
1802    format(/,' #  IC     XCOUT     XCIN  XCF/AV       RL',
     *              '        RP        dXC      dR')
1803     format(i4,a8,2(1pe10.3),0pf7.3,1x,(4(1pe10.3)))

c WRITE GAS-AQUEOUS SUMS  
c   MAKE XC,  RP, RL=GAS+AQ SUM for gas species.
c    CONVERT AQUEOUS XC TO GAS UNITS
c     (Note, sum aqueous into gas for xc, but not for xcout.
c      xcout is made gas-aqueous sum in postlump)
c       
        do j=1,c_nchem2
          if(c_npequil(j).eq.j) then
            xrr( kw,1) = 0.
            rpro( kw,1) = 0.
            rloss( kw,1) = 0.
            do  neq=1,(c_nequil(  j)+1)
              ic=j
              if(neq.gt.1) ic = c_ncequil(  j,(neq-1))
              alpha(  kw) = 1.
              if(neq.gt.1) alpha( 1)= c_h2oliq( 1)*avogadrl
                          rloss( kw,1) = rloss( kw,1)+c_rl( kw,ic)
                          rpro( kw,1)  = rpro( kw,1) +c_rp( kw,ic)
c GAS-AQUEOUS SUM + CONVERSION
c             xrr( kw,1)   = xrr( kw,1)  +c_xcout( kw,ic)*alpha( kw)
c AQUEOUS CONVERSION, NO GAS-AQ SUM
              xrr( kw,1)   = c_xcout( kw,ic)*alpha( kw) 
            enddo             !do  neq=1,(c_nequil(  j)+1)
            xrr( kw,1) = c_xcout( kw,j)   ! CORRECTION- XR IS GAS-AQ SUM
            beta(1)  = xrr( kw,1) -   c_xcin( kw,j)
            gamma(1) = rpro( kw,1) - rloss( kw,1)

            write(c_out,1803) j, c_tchem(j),
     *       xrr( kw,1),  c_xcin( kw,j), xcfinr(kw,j)
     *       ,rloss( kw,1), rpro( kw,1)
     *       ,beta(1), gamma(1)
          endif             !if(c_npequil(j).eq.j) then
        enddo            !do j=1,c_nchem2


c ------
c END SUMMARY WRITE OPTION
c ------


c GAS-PHASE REACTION RATES  
         write(c_out,1143)
1143     format(/' GAS-PHASE REACTION RATES')
         do      nr=1,c_nreac
          write(c_out,1144) nr, (c_treac(j,nr),j=1,5), c_rr(kw ,nr),
     *    ratek(kw ,nr)
1144      format(i4,2x,a8,'+',a8,'=>',a8,'+',a8,'+',a8,2(1pe10.3))
         enddo  

          write(c_out,1146)
1146       format(/,'HENRYS LAW CONSTANT + MODIFIED CONSTANT',
     *       ' WITH DIFFUSION ADJUSTMENT:')
          if(c_nreach.gt.0) then
            do i=1,c_nreach
               write(c_out,1147) (c_treach(j,i),j=1,2),
     *         rateh(kw ,i),rhdif(kw ,i)
            enddo
c           write(c_out,1147) ((c_treach(j,i),j=1,2),
c    *         rateh(kw ,i),rhdif(kw ,i),i=1,c_nreach)
1147       format(a8,'=',a8,2x,2(1pe10.3))
          endif

c AQUEOUS EQUILIBRIUM CONSTANTS
         write(c_out,1153)
1153     format(/,' AQUEOUS EQUILIBRIUM CONSTANTS (H+)')
         do  nrq = 1,c_nreacq
          write(c_out,1154) nrq, (c_treacq(j,nrq),j=1,3), 
     *    rateq(kw ,nrq)
1154      format(i4,2x,a8,'<=>',a8,'+',a8,2(1pe10.3))
         enddo  

c SPECIAL EQUILIBRIUM CONSTANTS
         write(c_out,1156)
1156     format(/,' SPECIAL EQUILIBRIUM CONSTANTS ')
         do  nrqq = 1,c_nreaqq
          write(c_out,1154) nrqq, (c_treaqq(j,nrqq),j=1,3), 
     *    rateqq(kw ,nrqq)
         enddo  


2000   return
       end


c --------------------------------------------------------------
