c=============================================================================
c
c $Id: quadchem_init.F,v 1.1 2003/05/16 22:22:25 akinorii Exp $
c 2007 MODIFICATION
c
c CODE DEVELOPER
c   Akinori Ito, U of M
c   modified by Sandy Sillman, U of M  (2007 version)
c
c FILE
c   quadchem_init.F
c
c ROUTINES
c   Do_Quad_Init
c
c ARGUMENTS
c   chem_infile_name : quadchem input file name
c   qj_infile_name : qj input file name
c
c NOTES
c  quad part based on box
c
c  2007 Modification:  WRITE OUTPUT       to check for dimensions
c
c=============================================================================
 
      subroutine Do_Quad_Init

      use h_simpchem_cmq
      use h_rbchemmech
      use h_imp_control
      use h_imp_dims
      use h_imp_gen_io

      implicit none

c#     include "simpchem_cmq.h"
c#     include "rbchemmech.h"
c#     include "imp_control.h"
c#     include "imp_dims.h"
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** From chemistry version
c#     include "imp_gen_io.h"
c ** End of modifications by Luis Olcese, UM
c ********************************************

c     --------------------------------
c     IMPACT-related values.
c     --------------------------------

c     integer :: lrin          !  Replaced with c_rin in common

c     ----------------
c     Begin execution.
c     ----------------

c Initial input for chemical solver.
      c_rin = 88
c Set write index for chemistry test output
      c_out = 43

c     ====================
      call Asc_Open_Rd
c     ====================
     &  (c_rin, chem_infile_name)


c     =================
      call chemread
c     =================
c    &  (c_rin)


      if (phot_opt == 6) then
        open(unit = 999, file = qj_infile_name, status = 'old')
c       ===========
        call Hvread
c       ===========
     &    (999, nhv, hvmat, hvmatb, jarray)
      end if

c     =================
      call Quadinit
c     =================

c 2007 Modification:  WRITE OUTPUT       to check for dimensions

      if(i1.eq.1.and.ju1.eq.1) then
        write(6    , 101)
        write(6    ,102) c_cdim, c_nchem1, c_nchem2, c_nh2o,
     &                           num_species, num_active,
     &                           num_chem, num_molefrac,
     &                           imgas_num

101     format(/,' QUADINIT: Chemistry initialization:',//,
     &    ' Note:  c_cdim >> c_nchem2 >> c_nchem1',/,
     &    ' c_nchem1 = num_species and  includes CO2.',/,
     &  ' num_species = num_active + 1, extra is num_imgas (density)',/,
     &  ' num_chem=num_species;  num_molefrac = num_active')

102     format(/, ' Species array dimensions:',/,
     &         '    c_cdim (quadchem array dimension)       =',i8,/,
     &         '    c_nchem1 (quadchem input/output species)=', i8,/,
     &         '    c_nchem2 (quadchem total species)       =',i8,/,
     &         '    c_nh2o   (quadchem H2O index    )       =',i8,/,
     &         '    num_species (IMPACT total species)      =',i8,/,
     &         '    num_active  (IMPACT active species)     =',i8,/,
     &         '    num_chem    (IMPACT total chem   )      =',i8,/,
     &         '    num_molefrac (IMPACT active mix ratios) =',i8,/,
     &         '    num_imgas   (IMPACT density index)      =',i8)

         write(6    , 104) c_rdim, c_nreac, num_qks
104      format(/,' Gas phase reaction dimensions:',/,
     &        ' c_rdim (quadchem max dimension)           =',i8,/,
     &        ' c_nreac ( quadchem number of reactions)   =',i8,/,
     &        ' num_qks (IMPACT number of reactions)      =',i8)

         write(6    , 105) num_qjs
105      format (/,' num_qjs (IMPACT number of jvals, shld=56) :',i8)

         write(6    , 106) c_hdim, c_nreach, num_qhs
106      format(/,' Henrys law        dimensions:',/,
     &        ' c_hdim (quadchem max dimension)           =',i8,/,
     &        ' c_nreach ( quadchem number of Henrys)    =',i8,/,
     &        ' num_qhs (IMPACT number of Henrys  )      =',i8)

         write(6    , 107) c_aqdim, c_nreacq, num_qaqs
107      format(/,' Aqueous equilbrium dimensions:',/,
     &        ' c_aqdim (quadchem max dimension)          =',i8,/,
     &        ' c_nreacq( quadchem number of Aq. equil)   =',i8,/,
     &        ' num_qaqs (IMPACT number of Aq. equil)     =',i8)

         write(6    , 108) c_aqqdim, c_nreaqq, num_qaqqs
108      format(/,' Special aq. equilibrium dimensions:',/,
     &        ' c_aqqdim (quadchem max dimension)         =',i8,/,
     &        ' c_nreaqq ( quadchem number of Sp. equil) =',i8,/,
     &        ' num_qaqqs (IMPACT number of Sp. equil)    =',i8)

        if( c_cdim.lt.c_nchem2
     &     .or. c_rdim.lt. c_nreac
     &     .or. c_hdim.lt.c_nreach
     &     .or. c_aqdim.lt.c_nreacq
     &     .or. c_aqqdim.lt.c_nreaqq
     &     .or.c_nchem1.gt.num_species
     &     .or.c_nreac.gt.num_qks
     &     .or.num_qjs.ne.56

     &   )   write(6,111)
111      format(/,' WARNING:  LIKELY ERROR IN CHEM DIMENSIONS!!!')

         write(6    ,121) c_kvec
121      format(/,'Chemistry vector dimension = ', i5)

         write(6    ,123) i1,i2,ju1, j2,k1,k2
123      format(/,' Local grid dimensions: i1, i2, ju1,j2,k1,k2=', 6i5)

         write(6    ,126) i1_gl,i2_gl,ju1_gl, j2_gl,k1_gl,k2_gl
126      format(/,' Global grid dimensions: i1, i2, ju1,j2,k1,k2=', 6i5)

      endif    !if(i1.eq.1.and.ju1.eq.1) then

      Close (c_rin)
      Close (999)

      return

      end
