c=============================================================================
c
c $Id: quadchem_solver.F,v 1.4 2002/08/27 21:11:10 akinorii Exp $
c
c CODE DEVELOPER
c   Akinori Ito, U of M
c   2007:  Modified by Sandy Sillman, U of M
c            with imported components of chemmain1.F
c
c FILE
c   quadchem_solver.F
c
c ROUTINES
c   Do_Quad_Solver
c   Strat_OH
c
c=============================================================================
c quadchem_solver091909.F - 
c  w/ optional HARD-WIRED sum of ev_SOA to gas transported spec.
c    at the end of chemistry
c 
c quadchem_solver062509.F  w/ EXPO DECAY CORRECTION
c quadchem_solver101307:  SUCCESSFUL VERSION
c  Version with allocated arrays moved from quadchem to quadchem_solver.
c 92307: VERSION WITH COMMON BLOCKS ELIMINATED (except rbchemmech.h)

c DEBUG WRITES - commented out execpt for:  
c   TEST WRITE (chemwrit)          (ON)
c   c_kkw OPTION for specific grid (OFF)
c   Ftiming -  remains.           (OFF) 
c
c DEBUG WRITES to identify do_trop_chem (only with pr_diag)
c LONG DEBUG WRITE - added to get fort.43 output. - CUT HERE.
c
c   Note:  cout = 6.
c   (for FULL OUTPUT see  VERSION_DEBUG92107)

c NOTE: lexpo option - yes.
 
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_Quad_Solver
c
c NOTE OPTION:  write for specific grids (ON)
c
c NOTE OPTION: set CO2, H2, CH4 and/or CO here.
c
c DESCRIPTION
c   This is the main control routine for the chemistry solver, 
c   "Quadchem" .  (rbchem, 2007)
c
c  2007 modifications:  new version of solver, 
c                     with features adopted from rbchemmain1.f
c  Enters input values from IMPACT, calls solver, 
c    and puts updated values back in IMPACT.
c
c  Note:  I/O includes rates (calculated elsewhere) and concentrations.
c         Rate calculation and I/O for rates (included in rbchemmain.f)
c          appears in chem_update.F,Qj_update.F, Qk_update.F, Rate_Quad.F.
c    
c FUTURE CHANGES/NEEDS: 
c    For aqueous version:  
c       bring LWC (c_h2oliq) into quadchem_solver, 
c       add  wet deposition (c_xcwdep) to quadchem.F
c       add wet deposition to main IMPACT array at end of quadchem_solver.F
c
c ARGUMENTS
c   chemintv         : chemistry time step (s)
c   const_nb, const  : species concentration, known at zone centers (molec/cm^3)
c   ratek_imp        : reaction rate constants (units vary)
c   ratek_cld_imp    : reaction rate constants of cloud reactions (molec/cm^3/s)
c   rateh_imp        : Henry's law coefficients (mole l-1 atm-1)
c   rateh_imp_temp   : Henry's law coefficients (units?)
c   rateq_imp        : Aqueous equilibrium constants (usually mole l-1)
c   rateqq_imp       : Special equilibrium constants (usually mole l-1)
c   lbssad           : removal rate coefficient (s-1)  (cut - no longer used).
c   do_trop_chem     : do tropospheric chemistry
c Added by Sandy Sillman, 2007:  water vapor and cloud liquid water. watv
c   watv_imp         :  water vapor, molec/cm3 (from Qk_update)
c   clwc_imp         :  cloud liquid water, g/cm3 (from Qk_update)
c
c  Added import:  surface emissions for use in expo. decay flag.
c   surf_emiss       :  surface emissions, molec/cm3/timestep (?)
c                       (already included in concentration -
c                       see emiss_update.F  Update_Emis_Inquadchem
c
c OUTPUT ARRAY:     ij=Lat il=Lon
c  const(idim,judim , kdim, icdim, 1)  = lon lat sigma chem month
c     lat(judim), lon(idim), sig(kdim), spe(icdim)  mon_dim
c   Dimensions:  46   72      46         84          1 or 3 (3 per file)
c    (contrast with read_40s.f:  144 long 90 lats)
c  GRID:
c    lat 1=south pole,  then 4 degree intervals.
c    long 5 degree intervals.
c    sigma - below.
c 
c OUTPUT OPTIONS AND  CHOICES:
c months:  1, 4, 7, 10  (1st month each file)
c vertical layers 1, 6, 9 (700, 500)
c longitude layers:  2 (Switz, Nor), 16 (India), 24 (China),
c 36 (Pacific), => chg to 38 for Hawaii, 158 W)
c          56 (US Ohio 78W)  64 (Atlantic, Greenland)
c latitude: 29 (India-Hong Kong), 33 (US 38N), 36 (Germany 50N),
c   39(Norway NW canada S Greenland 62N)    (43 N Greenl,Arct Oc, 78N)
c
c PRESSURE FOR SIGMA LEVELS:
c  Pressure = 993.9361, 971.3033, 929.932, 875.0725, 812.5187, 745.0255,
C   674.5325, 604.5396, 536.5463, 471.5529, 410.059, 352.5648, 301.5699,
c   258.0512, 220.3515, 187.1253, 157.9652, 132.8937, 111.8113, 94.1256,
c   79.32458, 66.96581, 56.66834, 48.1072, 41.00591, 35.02351, 29.88902,
c   25.49196, 21.76033, 18.53666, 15.66944, 13.1342, 10.90642, 8.962614,
c   7.280282, 5.836426, 4.609549, 3.579152, 2.725237, 2.027807,
c   1.468363,
c   1.028907, 0.6929407, 0.4439656, 0.2664833, 0.1469953 ;

c HTAP regions:  
c   NA  North America (code 103)        125W-60W        15N-55N
c   EU  Europe + North Africa (code 102)        10W-50 E        25N-65N
c   SA  S. Asia (code 100)   50 E-95 E  5N-35N
c   EA  E. Asia (code 101)   95E-160 E  15N-50N


c-----------------------------------------------------------------------------
      subroutine Do_Quad_Solver
     &   (chemintv, const_nb, ratek_cld_imp,
     &    ratek_imp, rateh_imp,rateh_imp_temp, rateq_imp, rateqq_imp,
     &    temp_imp, watv_imp, clwc_imp,
     &    surf_emiss, do_trop_chem,
c-------Guangxing Lin -09212011---for clouds and aqueous aerosol
c      reactions      
     &    clfrac, press3c,aer_water, londeg,latdeg,co2_in)
c-------Guangxing Lin -09212011---for clouds and aqueous aerosol
c      reactions      

c      use Ftiming_Dao
      
      use h_imp_dims, only: pr_diag, loc_proc, num_active, num_chem,
     & i1, i2, ju1,j2,k1,k2, num_species, num_qks,num_qks, num_qhs,
     & num_qaqs, num_qaqqs,ilo,ihi,julo,jhi, ju1_gl,j2_gl
      use h_imp_control, only: nymd, nhms,imgas_num, ih2o_num,ino_num,
     & iisoprene_num, num_time_steps
      use h_rbchemmech, only: c_kvec,  c_cdim, c_rdim, c_hdim, c_aqdim,
     & c_aqqdim, c_nchem1,c_nco2,c_out, c_kkw, c_nreac, c_nno,c_nh2o,
     & c_nreach, c_nreacq, c_nreaqq, c_nchem2, c_nh2, c_tchem
      use h_rbchemvars, only: c_time 
      use h_sulfquad
       use abortutils,    only: endrun
      implicit none

c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "rbchemmech.h"
c#     include "rbchemvars.h"
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c#     include "sulfquad.h"
c ** End of modifications by Luis Olcese, UM
c ********************************************

c#     include "ftel_chem.h"
c     --------------------------------
c     IMPACT-related values.
c     --------------------------------

      logical :: do_trop_chem(i1:i2, ju1:j2, k1:k2)

      real*8  :: chemintv
      real*8  :: const_nb  (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: ratek_imp (i1:i2, ju1:j2, k1:k2, num_qks)
      real*8  :: ratek_cld_imp (i1:i2, ju1:j2, k1:k2, num_qks)
      real*8  :: rateh_imp (i1:i2, ju1:j2, k1:k2, num_qhs)
      real*8  :: rateh_imp_temp (i1:i2, ju1:j2, k1:k2, num_qhs)
      real*8  :: rateq_imp (i1:i2, ju1:j2, k1:k2, num_qaqs)
      real*8  :: rateqq_imp(i1:i2, ju1:j2, k1:k2, num_qaqqs)

      real*8  :: temp_imp  (ilo:ihi, julo:jhi, k1:k2)  ! based on temp3 in qk_update
      real*8  :: watv_imp  (i1:i2, ju1:j2, k1:k2)
      real*8  :: clwc_imp  (i1:i2, ju1:j2, k1:k2)

      real*8  :: surf_emiss(i1:i2, ju1:j2, num_species)
      real*8  :: clfrac    (i1:i2, ju1:j2, k1:k2)
      real*8  :: press3c(ilo:ihi, julo:jhi, k1:k2) 
      real*8  :: aer_water  (i1:i2, ju1:j2, k1:k2)
       real*8  :: londeg    (ju1_gl:j2_gl) !just pcols
      real*8  :: latdeg    (ju1_gl:j2_gl)
      real*8  :: co2_in      

c-----Guangxing Lin----rbchemvars.h---------------------      
      real*8 ::  c_h2oliq(c_kvec)   ! LWC grams cm-3
cx       real*8 ::  c_h2oaer(c_kvec)   ! aerosol water grams cm-3
       logical :: c_lexpo(c_kvec)        ! Flag for expo decay solution
        real*8 ::  c_h2ogas(c_kvec)   ! H2O abs humid molec/cm3

       real*8 ::  c_h2oaer(c_kvec)   ! aerosol water grams cm-3
       real*8 ::  c_raer(c_kvec)     ! wet aerosol radius cm

      real*8 ::  c_droplet(c_kvec)        ! droplet radius, cm
       real*8 ::  c_temp(c_kvec)    ! temperature K
        real*8 ::  c_ohtest      ! test: dOH/OH or dOH/HO2
      real*8 ::  c_notest      ! test: dNO2/NO2
      integer :: c_iter                  ! chem. number of iterations
      real*8 ::  c_rainfr(c_kvec)   ! rainout fraction

c  **** NEED TO ADD HERE: 
c          LWC (bring in)
c          emis, altitude index for lexpo


c     ----------------------
c     Variable declarations. (IMPACT)
c     ----------------------

      logical, save :: first = .true.

      integer :: il, ij, ik, iq, iftel   ! Indices for IMPACT I/O

      integer :: nsec_jan1


c LOCAL VARIABLES (from rbchemmain)
c
      integer :: kk             ! vector loop  - in boxvars AND chemlocal
      integer :: ic             ! chemistry counter - in boxvars AND chemlocal
      integer :: icq            ! aqueous species counter
      integer :: neq,i            ! Aqueous link to gas species counter
c      real*8 ::  c_h2oaer(c_kvec)   ! aerosol water grams cm-3
      real*8 avogadrl ! Avogadro Number * lit/cm3
      PARAMETER (avogadrl=6.0221367D20)

c LOCAL VARIABLE FOR INTERNAL USE
      real*8 sum1, sum2,tempval

c -------------------------------------------------
c  From chemmain1.f:  
c  Declare and allocate chemistry I/O arrays (from chemarrays.h)
c   These will be passed to all subroutines
c                    and are included in 'chemarrays.h'
c   (Rate constants are called here,
c       include main rate constant arrays (from chemlarr.h)
c

c I-O variables
       real*8, allocatable ::
     &        c_xcin(  :,: )         ! concentr molec/cm3
       real*8, allocatable ::
     &        c_xcout(  :,: )         ! concentr, mol/cm3
       real*8, allocatable ::
     &        c_xcav(  :,: )          ! concentr, mol/cm3
       real*8, allocatable ::
     &        c_xcemit(  :,: )         ! emis, molec/cm3
c      real*8, allocatable ::
c    &        c_xcwdep(  :,: )         ! wet dep, molec/cm3
      real*8, allocatable ::
     &        c_rp(  :,: )         ! Production, molec/cm3
      real*8, allocatable ::
     &        c_rl(  :,: )         ! Loss, molec/cm3
      real*8, allocatable ::
     &        c_rr(  :,: )         ! Reaction rate m/cm3

      real*8, allocatable :: 
     &     xcfinr( :,:) ! ratio FINAL/AVG cncn

c Rate constants
      real*8, allocatable ::
     &       ratek(:,:)            ! Rate constants
      real*8, allocatable ::
     &       rateh(:,:)            ! Henry's law constants
      real*8, allocatable ::
     &       rateq(:,:)            ! Aqueous eq. constants
      real*8, allocatable ::
     &       rateqq(:,:)           ! Special eq. constants
      real*8, allocatable ::
     &       rhdif(:,:)           ! H. mod by diffusion
      real*8, allocatable ::
     &       egasaq(:,:)           ! Gas-to-aq transfer s-1

c Allocated varibles to pass to quadchem   (new add)
c  (moved from quadchem)
c Local concentration arrays
      real*8, allocatable :: xc(:,:)             ! concentration molec/cm3
      real*8, allocatable :: xclastq(:,:)        ! conc molec/cm3
      real*8, allocatable :: xcoutcld(:,:)       ! concentration from cloud reactions molec/cm3
      real*8, allocatable :: c_rrcld(:,:)        ! reaction rates from cloud reactions m/cm3                              
      real*8, allocatable :: c_rraer(:,:)        ! reaction rates from aqueous aerosol reactions m/cm3                              

c Local solution arrays
      real*8, allocatable :: cpm(:,:,:)          ! Cross-prod for multi
      real*8, allocatable :: cpro(:,:,:)         ! Cross-production
      real*8, allocatable :: history(:,:,:)      ! Cncn for each iter
      real*8, allocatable :: geomavg(:,:)        ! Geom. avg factor
      real*8, allocatable :: rlm(:,:)            ! Loss for multi-spec
      real*8, allocatable :: rlmulti(:)          ! Loss group sum for multi
      real*8, allocatable :: rloss(:,:)          ! Solver loss rate
      real*8, allocatable :: rloss1(:,:)         ! Interim rloss w/out cpro
      real*8, allocatable :: rlpair(:,:)         ! Loss rate for pair grp
      real*8, allocatable :: rpm(:,:)            ! Production rate for multi
      real*8, allocatable :: rpmulti(:)          ! Product'n multi group sum
      real*8, allocatable :: rppair(:,:)         ! Production for pair grp
      real*8, allocatable :: rpro(:,:)           ! Solver production rate
      real*8, allocatable :: rpro1(:,:)          ! Interim rpro w/out cpro
      real*8, allocatable :: rrp(:,:)            ! Interim production rate
      real*8, allocatable :: rrl(:,:)            ! Interim loss rate
      real*8, allocatable :: xrm(:,:)            ! Initial conc. for multi
      real*8, allocatable :: xrp(:,:)            ! Solver prior concentration
      real*8, allocatable :: xrppair(:)          ! Solver prior pair group cn.
      real*8, allocatable :: xrr(:,:)            ! Solver concentration
      real*8, allocatable :: xrrm(:,:)           ! Initial solution for multi
      real*8, allocatable :: xrrpair(:)          ! Solution for pair group sum

      real*8, allocatable ::  BX(:)     ! For back_Eulersolution AX=B
      real*8, allocatable ::  XX(:)     ! For back_Eulersolution AX=B
      real*8, allocatable ::  AX(:,:)   ! For back_Eulersolution AX=B

      real*8, allocatable ::  qaerwat(:,:,:)     ! aerosol water g/cm^3 
      real*8, allocatable ::  wetrad_aer(:,:,:)  ! wet aersol radius (m) after water uptake 
      real*8, allocatable ::  rh(:,:,:)  ! relative humidity 
      integer :: ierror(100),ierr_count

      
      real*8 :: min_raer, max_raer,rate_total,rate_total_before

          ierr_count = 0
         ierr_count = ierr_count+1

      Allocate (c_xcin  (c_kvec, c_cdim),stat=ierror(ierr_count))
      ierr_count = ierr_count+1

      Allocate (c_xcout (c_kvec, c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (c_xcav  (c_kvec, c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (c_xcemit(c_kvec, c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
c     Allocate (c_xcwdep(c_kvec, c_cdim))
         ierr_count = ierr_count+1
      Allocate (c_rp    (c_kvec, c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (c_rl    (c_kvec, c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (c_rr    (c_kvec, c_rdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1

      c_xcin = 0.0d0; c_xcout = 0.0d0
      c_xcav = 0.0d0; c_xcemit = 0.0d0
      c_rp = 0.0d0; c_rl = 0.0d0; c_rr = 0.0d0
c     c_xcwdep = 0.0d0


      Allocate (xcfinr  (c_kvec, c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1

      Allocate (ratek   (c_kvec, c_rdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rateh   (c_kvec, c_hdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rateq   (c_kvec, c_aqdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rateqq  (c_kvec, c_aqqdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rhdif   (c_kvec, c_hdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (egasaq  (c_kvec, c_hdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1

      xcfinr = 0.0d0
      ratek = 0.0d0; rateh = 0.0d0; rateq = 0.0d0
      rateqq = 0.0d0; rhdif = 0.0d0; egasaq = 0.0d0

c Allocate new variables - moved from quadchem
      Allocate (xc(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (xclastq(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (xcoutcld(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (c_rrcld (c_kvec, c_rdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (c_rraer (c_kvec, c_rdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1

      xc = 0.0; xclastq = 0.0
      xcoutcld=0.0; c_rrcld=0.0; c_rraer=0.0
c
      Allocate (cpm(c_kvec, c_cdim, c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (cpro(c_kvec,c_cdim, c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (history(c_kvec,c_cdim,400),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (geomavg(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rlm(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rlmulti(c_kvec),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rloss(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rloss1(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rlpair(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rpm(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rpmulti(c_kvec),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rppair(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rpro(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rpro1(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rrp(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rrl(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (xrm(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (xrp(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (xrppair(c_kvec),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (xrr(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (xrrm(c_kvec,c_cdim),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (xrrpair(c_kvec),stat=ierror(ierr_count))
         ierr_count = ierr_count+1

      Allocate (qaerwat(i1:i2,   ju1:j2,k1:k2),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (wetrad_aer(i1:i2,   ju1:j2,k1:k2),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (rh(i1:i2,   ju1:j2,   k1:k2),stat=ierror(ierr_count))
         ierr_count = ierr_count+1

c ALLOCATE AX, BX, XX HERE
      Allocate (BX(100),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (XX(100),stat=ierror(ierr_count))
         ierr_count = ierr_count+1
      Allocate (AX(100,100),stat=ierror(ierr_count))
         ierr_count = ierr_count+1

      
       if ( any( ierror(1:ierr_count) .ne. 0 ) ) then
           print*, ierror(1:ierr_count)
           call endrun("Error in allocation in Quadchem")

         end if

      cpm = 0.0d0; cpro = 0.0d0; history = 0.0d0
      geomavg = 0.0d0; rlm = 0.0d0; rlmulti = 0.0d0
      rloss = 0.0d0; rloss1 = 0.0d0; rlpair = 0.0d0
      rpm = 0.0d0; rpmulti = 0.0d0; rppair = 0.0d0
      rpro  = 0.0d0; rpro1 = 0.0d0; rrp = 0.0d0
      rrl = 0.0d0; xrm = 0.0d0; xrp = 0.0d0
      xrppair = 0.0d0; xrr = 0.0d0; xrrm = 0.0d0
      xrrpair = 0.0d0

      qaerwat=0.0d0; wetrad_aer=0.0d0;rh=0.0d0

      BX = 0.0d0
      XX = 0.0d0
      AX = 0.0d0



c     ----------------
c     Begin execution.
c     ----------------
 
c Set write index for chemistry test output
       c_out = 43
c      c_out = 6
 
       call Get_Sec_Jan1
     &  (nsec_jan1, nymd, nhms)

              write(c_out,811)  nsec_jan1
              write(c_out,*) "nymd=",nymd,"nhms=",nhms
c       if(nsec_jan1.eq.16815600.or.
c     $    nsec_jan1.eq.16819200 )then
c              write(44 ,811)  nsec_jan1
c       endif
811          format(/,'Start of step: nsec_jan1=',i8, 1pe15.6) 
c        write(6,*)nymd, nhms
               
!      if (pr_diag) then
!        Write (6,*) 'Do_Quad_Solver called by ', loc_proc
        Write (c_out,*) 'Do_Quad_Solver called by ', loc_proc

c DEBUG - DIAGNOSTIC TEST WRITE (if pr_diag)
!        write(6 ,*) 'QUADCHEM SOLVER i1 i2 ju1 j2 k1 k2=',
!     *       i1, i2, ju1, j2, k1, k2
        write(c_out ,*) 'QUADCHEM SOLVER i1 i2 ju1 j2 k1 k2=',
     *       i1, i2, ju1, j2, k1, k2

!        write(6,*) 'div_const, c_nchem1, c_nco2,  imgas_num =',
!     &         div_const, c_nchem1, c_nco2, imgas_num
!        write(6,*) 'num_species, num_active, num_chem =',
!     &              num_species, num_active, num_chem

        write(c_out,*) 'div_const, c_nchem1, c_nco2,  imgas_num =',
     &         div_const, c_nchem1, c_nco2, imgas_num
        write(c_out,*) 'num_species, num_active, num_chem c_cdim =',
     &              num_species, num_active, num_chem, c_cdim

!      end if


c Set index for full diagnostic write of chemistry = 0
      c_kkw = 0
      kk = 1
c     ==========
      if (first) then
c     ==========
        first = .false.
        if (num_species == 15) then  ! Initial input for simple chemistry
          c_time = 300.
        else
          c_time = chemintv 
        end if
c     ==========
      end if
c     ==========

c CALL TO IMPACT STRATOSPHERIC OH
c      if (do_ftiming) call Ftiming_On  ('Strat_OH')
c

c     =============
      call Strat_OH
c     =============
     &   (chemintv, const_nb,
     &    ratek_imp, do_trop_chem)

c      if (do_ftiming) call Ftiming_Off  ('Strat_OH')

c-------------Guangxing Lin-09232011-----------------
c to calculate the aerosol water and wet radius
      rh(:,:,:) = 1.0d0 - (373.15d0 / temp_imp(i1:i2,ju1:j2,:))

      rh(:,:,:) =
     &  1013.25d0 * Exp (13.3185d0 * rh(:,:,:)    -
     &                    1.9760d0 * rh(:,:,:)**2 -
     &                    0.6445d0 * rh(:,:,:)**3 -
     &                    0.1299d0 * rh(:,:,:)**4)

      rh(:,:,:) =
     &  const_nb(:,:,:,ih2o_num) /
     $  const_nb(:,:,:,imgas_num)*
     &  press3c(i1:i2,ju1:j2,:) / rh(:,:,:)

      rh(:,:,:) = Max (Min (rh(:,:,:), 1.0d0), 0.0d0)


      call calc_aero_wateruptake(const_nb, temp_imp, press3c,
     &     qaerwat, wetrad_aer,rh)  
        
      aer_water = qaerwat                 
            
c-------------Guangxing Lin-09232011-----------------

c ENTER DEFAULT VALUES FOR METEOROLOGICAL INPUT 
c   (unless updated during loop below)
c
c      Liquid water concentration, grams cm-3
c
           c_h2oliq( 1 ) = 0.
           c_h2oaer( 1 ) = 0.

c      Rainout fraction during time step for aqueous species

           c_rainfr( 1 ) = 0.   

c      Water droplet radius, cm  (default  .001 = 10 mcm)

c           c_droplet( 1)  = 0.001
c           c_droplet( 1)  = 1.0d-5 
c       OPTIONAL - ENTER EMISSIONS FOR THE TIME STEP (Defalt=0.)
c         c_xcemit(kk,ic) = OPTIONAL emissions component only, molec/cm3
c         Normally, zero.

c       (FUTURE OPTION:  This can be entered directly
c          and expo decay solution used in emission areas
c          but only if converted 
c           to represent emissions averaged over mixing altitude (2 km?).
c          Note this does not add to concentration, 
c           but is used to set time history internal to time step.)
c
croy this is not needed
croy            do ic= 1, c_nchem1
croy              c_xcemit( 1 ,ic) = 0.
croy            enddo

c
c      FLAG FOR EXPONENTIAL DECAY SOLUTION
c        Exponential decay solution should be used in remote locations
c          or in the free troposphere;   where emissions are zero.
c        Default is here.
c        UPDATED BELOW FOR SPECIFIC GRIDS

           c_lexpo(kk) = .FALSE.

c LOOP TO CALL CHEMISTRY FOR GRID LOCATIONS 
      
      max_raer=0.0d0
      min_raer=1.0d-5
      rate_total=0.d0
      rate_total_before=0.d0   
!=========debug======Guangxing Lin
      if(1.eq.1) then 
      do ik = k1, k2
        do ij = ju1, j2
          do il = i1, i2

c DEBUG - TEMPORARY TEST WRITE -  Identify do_trop_chem
c           if(pr_diag.and.il.eq.60.and.ij.eq.32)  then
c         if(il.eq.66.and. ij.eq. 2.and.ik.eq.15) then
        if((il.eq.1.and.ij.eq.1.and.ik.eq.1)) then
c             if(il.eq.66.and. ij.eq.3.and.ik.eq.1) then
c             write(6,*) 'QUADCHEM il ij k1,k2, do_trop =',
c     &       il, ij, k1,k2, do_trop_chem(il,ij,ik)
             write(c_out,*) 'QUADCHEM il ij ik do_trop =',
     &       il, ij, ik, do_trop_chem(il,ij,ik)
              c_kkw=0
c             c_kkw=1

            else
              c_kkw=0
           endif

           if (do_trop_chem(il,ij,ik)) then

c ENTER METEO. PARAMETERS THAT AFFECT CHEMISTRY: 
c FUTURE NEED:  
c
c     temperature (temp_imp = kel1 from chem_update, 
c                modeled after temp3 in qk_update)

           c_temp  ( 1 ) = temp_imp(il, ij, ik)
c
c     absolute humitidy (H2O concentration, molec cm-3

           c_h2ogas( 1 ) = watv_imp(il, ij, ik)
c
c      Liquid water concentration, grams cm-3
c      NOTE:  =0. unless flag 'do_wetchem' is on.
c
           c_h2oliq( 1 ) = clwc_imp(il,ij,ik)
c -------Guangxing Lin--06-13-2012----for test only----
c           c_h2oliq( 1 ) = clwc_imp(il,ij,ik)*0.2
c -------Guangxing Lin--06-13-2012----for test only----

c      Rainout fraction during time step for aqueous species
c       NOTE:  to use, integrate with other wet deposition
c              and add wet deposition output.

c          c_rainfr( 1 ) = 
c           if(c_rainfr( 1).lt.0)  c_rainfr( 1)=0.
c           if(c_rainfr( 1).gt.0.9999)c_rainfr( 1)=0.9999

c      Water droplet radius, cm  (default  .001 = 10 mcm)

          c_DROPLET( 1)  = 0.001

c       OPTIONAL - ENTER EMISSIONS FOR THE TIME STEP (Defalt=0.)
c         c_xcemit(kk,ic) = OPTIONAL emissions component only, molec/cm3
c
c         Note:  This does not affect species concentrations.
c          It only affects variation vs time within the time interval
c             in the exponential decay solution.
c
c         Emissions component MUST be smaller than the input c_xcin.
c          (If used, add WARNING algorithm from chemmain1.f)
c
c           do ic=1,nchem1
c             c_xcemit( 1 ,ic) = 0.  
c           enddo

c      FLAG FOR EXPONENTIAL DECAY SOLUTION
c        Exponential decay solution should be used in remote locations
c          or in the free troposphere;   where emissions are zero.

c        It can be used in locations if emissions are known.
c         Emissions must be "effective" emissions, including transport
c          (for example, in the daytime mixed layer,
c           several model layers are effected by emissions
c           on scales shorter than the simulated time interval)

c EXPONENTIAL DECAY OPTION

           c_lexpo(kk) = .FALSE.
           if(ino_num.le.div_const) ino_num = c_nno + div_const
           if(iisoprene_num.le.div_const)
     &            iisoprene_num =    30   + div_const
           if(ik.ge.5)                   c_lexpo(kk) = .TRUE. 
! Guangxing Lin--debug
           
           if(surf_emiss(il, ij, iisoprene_num).lt.1.d-5
     &         .and.surf_emiss(il, ij,ino_num).lt.1.d-5)
     &                                    c_lexpo(kk) = .TRUE.

c       FUTURE AQUEOUS OPTION:  
c       NaCl OPTION:  Set NaCl (molec/cm3) to a fixed moles/liter value
c         from input xnacl
c      (in chemmain1.f)

c ENTER SPECIES CONCENTRATIONS
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include the quadchem/sulfur chemistry
c                do ic = 1, num_active
c                  c_xcin(1, ic) = const_nb(il,ij,ik, ic)
                do ic = 1, c_nchem1
c ORIGINAL
c                 c_xcin(1, ic) = const_nb(il,ij,ik, ic + div_const)
c Modification to exclude nontransported species ( ev_... etc.). (Sandy.)
                  if(ic.lt.c_nco2.and.ic+div_const.lt.imgas_num) then
                    c_xcin(1, ic) = const_nb(il,ij,ik, ic + div_const)
                  else
                    c_xcin(1, ic) = 0.
                  endif
c End Modification to exclude nontransported species

c ** End of modifications by Luis Olcese, UM
c ********************************************
                end do

c   (end TELOMER OPTION control)
c             end if
c
c ENTER  H2O CONCENTRATION
              c_xcin(1,c_nh2o) =  c_h2ogas( 1 ) 
c
c OPTION:  SET CONCENTRATION FOR SPECIAL SPECIES 
c          ( CO2, H2, CH4, CO)
c 
c CO2 - set based on density 
croy: c_nco2 = 89 and imgas_num = 89??????
c c_xcin (1,c_cdim), c_cdim=211

c              c_xcin(1,c_nco2) =  0.880E+16
c              c_xcin(1,c_nco2) =  0.000375*const_nb(il,ij,ik,imgas_num)
              c_xcin(1,c_nco2) =  co2_in*const_nb(il,ij,ik,imgas_num)

c H2  - set based on density
c             c_xcin(1,c_nh2)  = 1.000E+13
c              c_xcin(1,c_nh2)  = 0.425e-06 *const_nb(il,ij,ik,imgas_num)
cGuangxing Lin, now it is set in flbc_set subroutine
c---------Guangxing Lin --10-21-2011-------add for aqueous SOA reactions-------
c O2  c_xcin(1,c_no2)
c              c_xcin(1,c_no2) =  0.21*const_nb(il,ij,ik,imgas_num)
c---------Guangxing Lin --10-21-2011-------add for aqueous SOA reactions-------

c CH4 c_xcin(1,c_nch4) = 0.240E+14
c             c_xcin(1,c_nch4)  = 1.400e-06 *const_nb(il,ij,ik,imgas_num)

c ENTER RATE CONSTANTS  (from earlier rate calculation)
croy: here ratek_imp has only num_active=88 species updated????

              do iq = 1, c_nreac
                ratek(1, iq) = ratek_imp(il,ij,ik, iq)
              enddo
              if(c_nreach.gt.0) then
              do iq = 1, c_nreach
                rateh(1, iq) = rateh_imp(il,ij,ik, iq)
c                rateh_temp(1, iq) = rateh_imp_temp(il,ij,ik, iq)
              enddo
              end if
              if(c_nreacq.gt.0) then
              do iq = 1,  c_nreacq
                rateq(1, iq) = rateq_imp(il,ij,ik, iq)
              enddo
              end if
              if(c_nreaqq.gt.0) then
              do iq = 1, c_nreaqq
                rateqq(1, iq) = rateqq_imp(il,ij,ik, iq)
              end do
              end if
c-----------------------Guangxing Lin--11-22-2011-------------------------
c-----this is for Ervens aq_SOA parameterazation only, 
c-----otherwise, turn it below off; it is risky, should modify later-------
                      c_xcin(1,116)=0.0
              c_xcin(1,117)=0.0
              c_xcin(1,121)=0.0
                do iq=683,695
                 ratek(1,iq)=0.0
               end do
               do iq=740,752
                 ratek(1,iq)=0.0
               end do
               ratek(1,683)=ratek_imp(il,ij,ik, 683)
               ratek(1,685)=ratek_imp(il,ij,ik, 685)

               ratek(1,697)=0.0
               ratek(1,698)=0.0
               ratek(1,754)=0.0
               ratek(1,755)=0.0
               ratek(1,761)=0.0
               ratek(1,762)=0.0
               ratek(1,768)=0.0
               ratek(1,769)=0.0
               ratek(1,771)=0.0
               ratek(1,772)=0.0
               ratek(1,821)=0.0
               ratek(1,822)=0.0
               rateh(1,2 )=rateh(1,2)*4.19d5/5.8
               rateh(1,3 )=rateh(1,3)*3.71d3/1.4
c-----------------------Guangxing Lin--11-22-2011-------------------------

c DEBUG OPTION:  SET CHEM. OUTPUT FOR DIFFICULT FILES
c  914 error:  28 29  11; 29 30 11; 30 31 11; 30 32 11, 31 33 11, 28 29 12,...

c      if(il.eq.60.and.ij.eq.32.and.ik.eq.1) c_kkw=1    !92407 AHO2 error
c      if(il.eq.3.and.ij.eq.2.and.ik.eq.1) c_kkw=1    !92407 AHO2 error

c       if (nsec_jan1.eq.25797600.and.il.eq.3.and.ij.eq.2.and.ik.eq.1) 
c      if(il.eq.56.and.ij.eq.33.and.ik.eq.1)     !92407 AHO2 error
c     $   c_kkw=1

      if(c_kkw.gt.0) write(6,821) il, ij, ik, c_kkw
      if(c_kkw.gt.0) write(c_out,821) il, ij, ik, c_kkw
821   format(/,' Before QUADCHEM: il ij ik c_kkw=',4i5)

c DEBUG
c      write(44,*) loc_proc, il,ij,ik
c      write(6,*) 'BEFORE QUADCHEM', loc_proc
c      write(6,*) il, ij, ik

c DEBUG WRITE FOR A-DI ne_oSOA ERROR
!       if((il.eq.1.and.ij.eq.1.and.ik.eq.1)) then
!       write(6,*) 'BEFORE QUADCHEM-step0: il, ij, ik=', il,ij,ik 
!         write(6,*) ' CHEM    XCIN  XCOUT   RP     RL   ' 
!         write(6,305) c_tchem(11),  c_xcin(1,11), 
!     &     c_xcout(1,11), c_rp(1,11), c_rl(1,11) 
!       endif

c CALL THE DRIVER SUBROUTINE FOR THE CHEMSTRY SOLUTION
               sum1=0.0
c               tempval =1.0d15
     
c       if(nsec_jan1.eq.16815600.or.
c     $    nsec_jan1.eq.16819200 )then
c                 write(44,*)'testNaN value before quadchem 
c     $             for  il,ij,ik'
c
c     $              ,il,ij,ik
         
               do ic =1, c_nchem1-1
c Test write for NaN
        if(c_xcin(1,ic).ne.c_xcin(1,ic))then
c           write(6,*)'tempval',tempval
c                 write(6,*)'testNaN value before quadchem 
c     $             for c_xcin,ic, il,ij,ik'
c i    $              ,c_xcin(1,ic),ic,il,ij,ik
c                  exit
        endif
c                 write(44,822) ic, c_xcin(1,ic) 
c822          format('ic, conc, ', i5,1pe15.6) 
                  sum1=sum1+c_xcin(1, ic)
                
              end do          
c                if (sum1 .ne.sum1) then
c                end if
c        endif 
!Guangxing Lin  debug
!       if(c_h2oliq(1).gt.1.d-17.and.clfrac(il,ij,ik).gt.1.d-3) then
       if(c_h2oliq(1).gt.1.d-20) then
        
c              if (do_ftiming) call Ftiming_On  ('quadchem')
c             ====================
              call quadchem
c             ====================
     &          (c_xcin, c_xcout, c_xcav, c_xcemit, 
c    &                                        c_xcwdep,
     &           c_rp, c_rl, c_rr ,
     &           xcfinr,
     &           ratek, rateh, rateq, rateqq, rhdif, egasaq
     &          , c_temp,  c_h2oliq ,   c_rainfr,  c_droplet, c_lexpo
     &          , c_ohtest, c_notest, c_iter


     &    ,xc, xclastq
     &    ,cpm, cpro, history, geomavg, rlm, rlmulti, rloss, rloss1
     &    ,rlpair, rpm, rpmulti, rppair, rpro, rpro1, rrp, rrl
     &    ,xrm, xrp, xrppair, xrr, xrrm, xrrpair, AX, BX, XX

     &          )

c              if (do_ftiming) call Ftiming_Off  ('quadchem')
c-----------------------GuangxingLin--09082011----------------
c------aquaeous aerosol reactions

                  xcoutcld=c_xcout
                  c_rrcld=c_rr
        
                    
c       if((il.eq.1.and.ij.eq.2.and.ik.eq.1)) then
       if((londeg(ij).lt.281.0.and.londeg(ij).gt.279.0.and.
     &   latdeg(ij).lt.41.0.and.latdeg(ij).gt.40.0.and.ik.eq.2)) then
c       if((1.eq.1)) then
c       if  (il.eq.66.and. ij.eq. 2.and.ik.eq.15) then
c       write(6,*) 'after QUADCHEM-step1: il, ij, ik=', il,ij,ik 
c         write(6,*) ' CHEM    XCIN  XCOUT   RP     RL   ' 
c         write(6,305) c_tchem(11),  c_xcin(1,11), 
c     &     c_xcout(1,11), c_rp(1,11), c_rl(1,11) 
c         write(6,2244)clfrac(il,ij,ik),c_rr(1,644), il,ij,ik
         write(c_out, 201) il, ij, ik
201      format(/'TEST CHEMICAL OUTPUT in clouds: 
     $  LON, LAT, SIG il,ij,ik= ',3i3)
         write(c_out,*) 'secs= ',nsec_jan1,'num_step= ',
     &  num_time_steps, 'proc= ',loc_proc
          write(c_out, 303) c_temp(1), c_h2oliq(1), c_h2ogas(1)
303      format(/,'  temp, CLWC, WATV =',(3(1pe10.3)) )
                 call chemwrite(1,
     &      c_xcin,  c_xcout,
     &      xcfinr,  rateh, rhdif, ratek, rateq, rateqq,
     &       c_rr, c_rp, c_rl
c     &       xrr, rpro, rloss)
     &     , c_h2oliq, c_temp, c_iter 
     &     )  
       endif
       else
                 xcoutcld=0.0
                 c_rrcld=0.0
                 clfrac(il,ij,ik)=0.0d0 !Guangxing Lin-debug
       end if

        
               c_h2oaer(1) = qaerwat(il,ij,ik)
               !c_h2oaer(1) = 0.0d0 !debug only, Guangxing
               c_raer (1) = wetrad_aer(il,ij,ik)*100.d0  ! convert m to cm
              ! c_raer (1) = 0.001d0 !debug only 
             rhdif=0.d0
             egasaq =0.d0
             c_rr=0.d0
             do iq = 1, c_nreach

              rateh(1,iq) = rateh_imp_temp(il,ij,ik, iq)*c_h2oaer(1)
             end do
               rateh(1,2 )=rateh(1,2)*4.19d5/5.8
               rateh(1,3 )=rateh(1,3)*3.71d3/1.4
              c_xcin(1,116)=0.0
              c_xcin(1,117)=0.0
              c_xcin(1,121)=0.0
!       if((il.eq.56.and.ij.eq.33.and.ik.eq.16)) then
!             write(6,665) rateh(1,2)
665      format(/,'  rateh aerosol_o2 =',((1pe10.3)) )
!       end if
c-----------------------Guangxing Lin--11-22-2011-------------------------
c-----this is for Ervens aq_SOA parameterazation only, 
c-----otherwise, turn it below off; it is risky, should modify later-------
c               do iq=683,692
c                 ratek(1,iq)=ratek_imp(il,ij,ik, iq)
c               end do
c               do iq=740,752
c                 ratek(1,iq)= ratek_imp(il,ij,ik,iq)
c               end do
c               ratek(1,693)=const_nb(il,ij,ik,49)/1.0d7
c     &          *ratek_imp(il,ij,ik, 693)
c               ratek(1,694)=const_nb(il,ij,ik,49)/1.0d7
c     &           *ratek_imp(il,ij,ik, 694)
c               ratek(1,695)=const_nb(il,ij,ik,49)/1.0d7
c     &           *ratek_imp(il,ij,ik, 695)
c               ratek(1,752)=ratek_imp(il,ij,ik,752)
c               ratek(1,697)=ratek_imp(il,ij,ik,697)
c               ratek(1,698)=ratek_imp(il,ij,ik,698)
c               ratek(1,754)=ratek_imp(il,ij,ik,754)
c               ratek(1,755)=ratek_imp(il,ij,ik,755)
c               ratek(1,761)=ratek_imp(il,ij,ik,761)
c               ratek(1,762)=ratek_imp(il,ij,ik,762)
c               ratek(1,768)=ratek_imp(il,ij,ik,768)
c               ratek(1,769)=ratek_imp(il,ij,ik,769)
c               ratek(1,771)=ratek_imp(il,ij,ik,771)
c               ratek(1,772)=ratek_imp(il,ij,ik,772)
               ratek(1,821)=ratek_imp(il,ij,ik,821)
               ratek(1,822)=ratek_imp(il,ij,ik,822)
               ratek(1,684)=ratek_imp(il,ij,ik, 684)
               ratek(1,685)=0.0
               ratek(1,683)=0.0
c-----------------------Guangxing Lin--11-22-2011-------------------------

                 
               c_h2oliq(1) = c_h2oaer(1)
               c_droplet(1) = c_raer(1)
              if(c_raer(1).le.min_raer) then
                  min_raer=c_raer(1)
              elseif(c_raer(1).ge.max_raer) then 
                  max_raer=c_raer(1)
              end if 
c               if(c_droplet(1).le.1d-10) then
c                c_droplet(1)=1.0d-7
c                c_h2oliq(1)=0.0d0
c               end if 
c               c_h2oliq(1) = 1.0d-10 
c               c_droplet(1) = 1.0d-4
               
c             ====================
              call quadchem
c             ====================
     &          (c_xcin, c_xcout, c_xcav, c_xcemit, 
c    &                                        c_xcwdep,
     &           c_rp, c_rl, c_rr ,
     &           xcfinr,
     &           ratek, rateh, rateq, rateqq, rhdif, egasaq
     &          , c_temp,  c_h2oliq ,   c_rainfr,  c_droplet, c_lexpo
     &          , c_ohtest, c_notest, c_iter


     &    ,xc, xclastq
     &    ,cpm, cpro, history, geomavg, rlm, rlmulti, rloss, rloss1
     &    ,rlpair, rpm, rpmulti, rppair, rpro, rpro1, rrp, rrl
     &    ,xrm, xrp, xrppair, xrr, xrrm, xrrpair, AX, BX, XX

     &          )
c       if((il.eq.56.and.ij.eq.33.and.ik.eq.2)) then
c       if((1.eq.1)) then
c       if((il.eq.1.and.ij.eq.2.and.ik.eq.1)) then
       if((londeg(ij).lt.281.0.and.londeg(ij).gt.279.0.and.
     &   latdeg(ij).lt.41.0.and.latdeg(ij).gt.40.0.and.ik.eq.2)) then
c        il=56
c            ij=33
c          if(i1.le.il.and.i2.ge.il.and.ju1.le.ij.and.j2.ge.ij
c     &         .and.k1.le. 2             ) then

c       if  (il.eq.66.and. ij.eq. 2.and.ik.eq.15) then
c       write(6,*) 'after QUADCHEM-step2: il, ij, ik=', il,ij,ik 
c         write(6,*) ' CHEM    XCIN  XCOUT   RP     RL   ' 
c         write(6,305) c_tchem(11),  c_xcin(1,11), 
c     &     c_xcout(1,11), c_rp(1,11), c_rl(1,11) 
!         write(6,2244)clfrac(il,ij,ik),c_rr(1,644), il,ij,ik
2244     format(/'gxlin-cldfrac= ',2f9.3,3i3)
!         write(6,665) rateh(1,2)
c665      format(/,'  rateh aerosol_o2 =',((1pe10.3)) )
         write(c_out, 202) il, ij, ik
202      format(/'TEST CHEMICAL OUTPUT in aeroosols: 
     &   LON, LAT, SIG il,ij,ik= ',3i3)
         !write(c_out,*) nsec_jan1
         write(c_out,*) 'secs= ',nsec_jan1,'num_step= ',
     &  num_time_steps, 'proc= ',loc_proc
         write(c_out,665) rateh(1,2)
          write(c_out, 403) c_temp(1), c_h2oaer(1), c_h2ogas(1)
403      format(/,'  temp, aerosol water, WATV =',(3(1pe10.3)) )
                 call chemwrite(1,
     &      c_xcin,  c_xcout,
     &      xcfinr,  rateh, rhdif, ratek, rateq, rateqq,
     &       c_rr, c_rp, c_rl
c     &       xrr, rpro, rloss)
     &     , c_h2oliq, c_temp, c_iter 
     &     )  
       endif

c      if (clfrac(il,ij,ik).lt.0.0.or.clfrac(il,ij,ik).gt.1.0)then
c         write(6,2244)clfrac(il,ij,ik), il,ij,ik
c2244     format(/'gxlin-cldfrac= ',f9.3,3i3)
c         write(c_out,2244)clfrac(il,ij,ik), il,ij,ik
c      end if
                do ic = 1, c_nchem2
                 c_xcout(1,ic)=clfrac(il,ij,ik)*xcoutcld(1,ic)
     &            + (1-clfrac(il,ij,ik))*c_xcout(1,ic)
                end do
                
                do iq = 1, c_nreac
                 c_rraer(1,iq)=c_rr(1,iq)
                 c_rr(1,iq)   =clfrac(il,ij,ik)*c_rrcld(1,iq)
     &            + (1-clfrac(il,ij,ik))*c_rraer(1,iq)
                end do

c------------------------GuangxingLin--09082011-end--------------        
c DEBUG
c      write(45,*)loc_proc, il,ij,ik
c      write(6,*) 'AFTER  QUADCHEM', loc_proc
c      write(6,*) il, ij, ik
c DEBUG WRITE
c        if(pr_diag.and.il.eq.57.and.ij.eq.29.and.ik.eq.1)
c     &     write(6,*) 'After quadchem: loc_proc=',
c     &     loc_proc
 
c DEBUG WRITE FOR A-DI ne_oSOA ERROR
c       if((il.eq.-56.and.ij.eq.33.and.ik.eq.1)
c     &     .or.(il.eq.-60.and.ij.eq.33.and.ik.eq.1) ) then
c       write(c_out,*) 'AFTER  QUADCHEM: il, ij, ik=', il,ij,ik 
c       write(c_out,*) 'xcin(52) = ', div_const,
c     &         c_xcin(1,52) 
c       write(c_out,*) 'xcin(193, 194) =', c_xcin(1,193),
c     &         c_xcin(1,194) 
c       write(c_out,*) 'xcout(52, 193, 194) = ', c_xcout(1, 52)
c     &    , c_xcout(1,193), c_xcout(1,194) 
c       endif

c ENTER SPECIES CONCENTRATION OUTPUT INTO IMPACT ARRAY 
c c             do ic = 1, c_nchem1
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include the quadchem/sulfur chemistry
c ******
c  Modified by Sandy Sillman 9-19-09 to do the following:
c    Only return transported species to IMPACT array (ic<nco2)
c ******
c              do ic = 1, c_nchem1-1
c                if(ic.ne.imgas_num) then
cc c            write(c_out,*) ic, const_nb(il,ij,ik, ic),c_xcout(1, ic)
c                const_nb(il,ij,ik, ic) = c_xcout(1, ic)
c            sum1=0.
              do ic = 1, c_nchem1 - 1
c               if(ic + div_const.ne.imgas_num) then
                if(ic + div_const.le.imgas_num) then
                  const_nb(il,ij,ik, ic + div_const) = c_xcout(1, ic)

c
c ** End of modifications by Luis Olcese, UM
c ********************************************
                endif
              end do
   
c       if(nsec_jan1.eq.16815600.or.
c     $    nsec_jan1.eq.16819200 )then
c                 write(44,*)'testNaN value after quadchem 
c     $             for  il,ij,ik'
c     $              ,il,ij,ik
c               sum1=0.0 
c               do ic =1, c_nchem1-1
c Test write for NaN
c        if(c_xcout(1,ic).ne.c_xcout(1,ic))then
c           write(6,*)'tempval',tempval
c                 write(6,*)'testNaN value after quadchem 
c     $             for c_xcout,ic, il,ij,ik'
c     $              ,c_xcout,ic,il,ij,ik
c                  exit
c        endif
c                 write(44,822) ic, c_xcin(1,ic) 
c                  sum1=sum1+c_xcout(1, ic)
                
c              end do          
cic                if (sum1 .ne.sum1) then
c                 write(c_out,*)'testNaN value after quadchem 
c     $             for  il,ij,ik'
c     $              ,il,ij,ik
c                endif
c         endif 
c              do ic =1, c_nchem1-1
c Test write for NaN
c                  sum1=sum1+c_xcout(1, ic)
cic                if (c_xcout(1, ic) /= c_xcout(1, ic)) then
c                 write(6,*)'testNaN value after quadchem 
c     $               for ic, il,ij,ik'
c     $              ,ic,il,ij,ik
c                end if
c                exit
c              end do          
c
c  Modified by Sandy Sillman 9-19-09 : 
c    For nontransported ev_SOA, add SOA to transported gas species
c     (HARD-WIRED CHEMISTRY here)

c HARD-WIRED OPTION: 
c      if(c_nco2.eq.109.and.c_nchem1.ge.130) then 
c SKIP HARD-WIRED OPTION: 
       if(c_nco2.eq.-999) then
         const_nb(il,ij,ik, 52 + div_const) = 
     &   const_nb(il,ij,ik, 52 + div_const) + c_xcout(1, 110)
         const_nb(il,ij,ik, 83 + div_const) = 
     &   const_nb(il,ij,ik, 83 + div_const) + c_xcout(1, 111)
         const_nb(il,ij,ik, 93 + div_const) = 
     &   const_nb(il,ij,ik, 93 + div_const) + c_xcout(1, 112)
         const_nb(il,ij,ik, 96 + div_const) = 
     &   const_nb(il,ij,ik, 96 + div_const) + c_xcout(1, 113)
         const_nb(il,ij,ik, 97 + div_const) = 
     &   const_nb(il,ij,ik, 97 + div_const) + c_xcout(1, 114)
         const_nb(il,ij,ik, 98 + div_const) = 
     &   const_nb(il,ij,ik, 98 + div_const) + c_xcout(1, 115)
         const_nb(il,ij,ik, 94 + div_const) = 
     &   const_nb(il,ij,ik, 94 + div_const) + c_xcout(1, 116)
         const_nb(il,ij,ik ,99 + div_const) = 
     &   const_nb(il,ij,ik ,99 + div_const) + c_xcout(1, 117)
         const_nb(il,ij,ik,100 + div_const) = 
     &   const_nb(il,ij,ik,100 + div_const) + c_xcout(1, 118)
         const_nb(il,ij,ik, 85 + div_const) = 
     &   const_nb(il,ij,ik, 85 + div_const) + c_xcout(1, 119)
         const_nb(il,ij,ik, 84 + div_const) = 
     &   const_nb(il,ij,ik, 84 + div_const) + c_xcout(1, 120)
         const_nb(il,ij,ik,101 + div_const) = 
     &   const_nb(il,ij,ik,101 + div_const) + c_xcout(1, 121)
         const_nb(il,ij,ik,102 + div_const) = 
     &   const_nb(il,ij,ik,102 + div_const) + c_xcout(1, 122)
         const_nb(il,ij,ik, 88 + div_const) = 
     &   const_nb(il,ij,ik, 88 + div_const) + c_xcout(1, 123)
         const_nb(il,ij,ik, 81 + div_const) = 
     &   const_nb(il,ij,ik, 81 + div_const) + c_xcout(1, 124)
         const_nb(il,ij,ik,103 + div_const) = 
     &   const_nb(il,ij,ik,103 + div_const) + c_xcout(1, 125)
         const_nb(il,ij,ik, 89 + div_const) = 
     &   const_nb(il,ij,ik, 89 + div_const) + c_xcout(1, 126)
         const_nb(il,ij,ik, 95 + div_const) = 
     &   const_nb(il,ij,ik, 95 + div_const) + c_xcout(1, 127)
         const_nb(il,ij,ik, 90 + div_const) = 
     &   const_nb(il,ij,ik, 90 + div_const) + c_xcout(1, 128)
         const_nb(il,ij,ik, 91 + div_const) = 
     &   const_nb(il,ij,ik, 91 + div_const) + c_xcout(1, 129)
         const_nb(il,ij,ik, 92 + div_const) = 
     &   const_nb(il,ij,ik, 92 + div_const) + c_xcout(1, 130)
       endif      ! if(c_nco2.eq.109) then 
c End of HARD-WIRED CHEMISTRY for ev_SOA

c DEBUG WRITE: identify PIP error (81)
c        if(c_xcout(1, 81).gt.1.0e10
c     &      .or. c_xcout(1,81).gt.100*c_xcin(1,81)
c     &           .and.c_xcin(1,81).gt.0.  ) then
c          write(c_out,941) il,ij,ik, c_xcin(1,81),c_xcout(1,81)
c941     format(' QUADCHEM SOA: il ij ik PIPin, PIPout =',
c     &     3i5,2(1pf10.3))
c           write(6,941) il,ij,ik, c_xcin(1,81),c_xcout(1,81)
c        endif
        
c        if(c_xcout(1,104).gt.1.0e10
c     &      .or. c_xcout(1,104).gt.100*c_xcin(1,104)
c     &        .and.c_xcin(1,104).gt.0. ) then

c          write(c_out,942) il,ij,ik, c_xcin(1,104),c_xcout(1,104)
c942     format(' QUADCHEM SOA: il ij ik SOAin, SOAout =',
c     &     3i5,2(1pf10.3))
c           write(6,942) il,ij,ik, c_xcin(1,104),c_xcout(1,104)

c        endif

c

c
c
c              if (sum1.ne.sum1) write(6,*)'NaN at', il, ij, ik
c LONG DEBUG WRITE - to generate long written output.     
c          if (pr_diag) then
c             write(c_out, 951) il, ij, ik, const_nb(il,ij,ik, 1 )
c             write(6 , 951) il, ij, ik, const_nb(il,ij,ik, 1 )
c 951           format(' AFTER QUADCHEM: il ij ik, O3=', 3i5,1pe10.3)
c          endif

c DEBUG WRITE
c        if(pr_diag.and.il.eq.57.and.ij.eq.29.and.ik.eq.1)
c       if  (il.eq.66.and. ij.eq. 2.and.ik.eq.15) then 
c              rate_total_before=ratek_imp(il,ij,ik,644)
c      write(6,*) 'Before ENTER REACTION:c_nreac,num_qks,ratek =',
c     &     loc_proc, c_nreac, num_qks, ratek_imp(il,ij,ik,644)
c      write(c_out,*)'Before ENTER REACTION:c_nreac,num_qks,ratek =',
c     &     loc_proc, c_nreac, num_qks, ratek_imp(il,ij,ik,644)
c       end if

c ENTER REACTION RATES (rr) IN PREVIOUS RATE CONSTANT ARRAY

              do iq = 1, c_nreac
                ratek_imp(il,ij,ik,iq) = c_rr(1,iq)
c                ratek_cld_imp(il,ij,ik,iq) = c_rr(1,iq)
                ratek_cld_imp(il,ij,ik,iq)=
     &                clfrac(il,ij,ik)*c_rrcld(1,iq)
              end do
c       if  (il.eq.66.and. ij.eq. 2.and.ik.eq.15) then 
c              rate_total=ratek_imp(il,ij,ik,644) 
c      write(6,*) 'after ENTER REACTION: c_nreac,num_qks,ratek =',
c     &     loc_proc, c_nreac, num_qks, ratek_imp(il,ij,ik,644)
c      write(c_out,*) 'after ENTER REACTION: c_nreac,num_qks,ratek =',
c     &     loc_proc, c_nreac, num_qks, ratek_imp(il,ij,ik,644)
c        end if
c DEBUG WRITE
c        if(pr_diag.and.il.eq.57.and.ij.eq.29.and.ik.eq.1)
c     &     write(6,*) 'After  ENTER REACTION RATES:',
c     &     loc_proc, il, ij, ik



c FUTURE:  WET DEPOSITION, H+ CONCENTRATION AND DEPOSITION HERE
c  (from rbchemmain1.f - if aqueous chemitry and deposition is included)

c -------------
c TEST WRITE OUTPUT (DEBUG OPTIONS)
c ------------
c
c OPTION TO WRITE FOR DIFFICULT CONVERGENCE
c      if(c_iter.ge.21) then

c ORIGINAL OPTION
c       if(c_ohtest.gt.0.2) then

c MODIFIED OPTION - SHORTER OUTPUT AND HIGH-NOX ERROR TEST
c        if( c_ohtest.gt.1.0.or.c_notest.gt.1.0
c     &    .or.( c_xcout(1,2)+c_xcout(1,3).gt.5e+11
c     &         .and. ( (c_xcout(1,2)+c_xcout(1,3)).gt.
c     &              2.*(c_xcin(1,2)+c_xcin(1,3))  )       )
c     &    ) then

c          write(c_out,211 ) il, ij, ik, c_iter
c211       format(/,'DIFFICULT CONVERGENCE: il, ij, ik, iter=', 4i5)
c          write(c_out,212) c_ohtest, c_notest
c212       format(' ohtest, notest = ', 2(1pe10.3))
c          write(c_out,213) c_xcout(1,  1)
c     &                 , c_xcout(1,  2), c_xcout(1,  3)
c     &                 , c_xcout(1,  9), c_xcout(1, 10)
c          write(c_out,213) c_xcin (1,  1)
c     &                 , c_xcin (1,  2), c_xcin (1,  3)
c213     format(' O3 NO NO2 OH HO2=', 5(1pe10.3))  
c        endif         !if(c_ohtest.gt.0.2) then
c      endif       !if(c_iter.ge.21) then
      
c TEST WRITE: 
c OPTION TO WRITE OUTPUT FOR SPECIFIC GRID
c   and/or time step (future)
c
c  HERE:  assume 4 x 5 grid;  
c      46 latitudes (south to north, 4 deg each, 2 deg at poles)
c      72 longitudes (0-4 degrees first)
c  (2x2.5:  91, 144)
c GER:  5-10, 48-52. lat 36, lon 2.
c NY/PENNA:  75-80 W (280-285), 40-44, lat 34, lon 57
c AZORES:  28 w (330-335) 37 N (36-40):  lat 33, lon 65
c
c Sandy's additional TEST WRITE for ev_IAP/IAP

c       if(
c     &     pr_diag.and.(
c NONE (replaces pr_diag)
c     &     ik.eq.-999.and.(
c
c     &         (il.eq.29.and. ij.eq.9.and.(ik.eq.4            ) )
c OHIO
c     &         (il.eq.56.and. ij.eq.33.and.(ik.eq.4            ) )
c GERMANY
c    &    .or. (il.eq. 2.and. ij.eq.36.and.(ik.eq.1            ) )
c SIBERIA/BAIKAL 55 N 100 E
c     &    .or. (il.eq.20.and. ij.eq.37.and.(ik.eq.1            ) )
c SOUTHERN BRAZIL/PARAGUAY/  25 S 60 W
c     &    .or. (il.eq.60.and. ij.eq.17.and.(ik.eq.1            ) )
c NORTH ATLANTIC
c     &    .or. (il.eq.60.and. ij.eq.33.and.(ik.eq.1            ) )

c     &                       )   ! pr_diag close     
c     &               )) then

c write:
c m0 in qk_update.F

         
c         write(6, 301) il, ij, ik
c301      format(/'TEST H2O2-step3: LON, LAT, SIG il,ij,ik= ',
c     &                     3i3)
c         write(6, 203) c_temp(1),c_h2oliq(1),c_droplet(1)
c     $   ,clfrac(il,ij,ik),rh(il,ij,ik)
c203      format(/,'  temp, WATER_aer, RADIUS_aer,cldfraction , rh=',
c     &    (5(1pe10.3)) )

c         write(c_out,*) "c_lexpo = ", c_lexpo(kk)
c         write(c_out,*) "iisoprene_num=", iisoprene_num
c         write(c_out,*) "ino_num = ", ino_num
c         write(c_out,*) "surf_emiss(il, ij, iisoprene_num) = ",
c     &       surf_emiss(il, ij, iisoprene_num)
c         write(c_out,*) "surf_emiss(il, ij,ino_num) =",
c     &       surf_emiss(il, ij,ino_num)

c         write(6,*) ' CHEM    XCIN  XCOUT   RP     RL   ' 
c         write(6,305) c_tchem(11),  c_xcin(1,11), 
c     &     c_xcout(1,11), c_rp(1,11), c_rl(1,11) 
c         write(c_out,305) c_tchem(118),  c_xcin(1,118), 
c     &     c_xcout(1,118), c_rp(1,118), c_rl(1,118) 
305      format(a8,6(1pe10.3))
c         write(c_out, 307) c_rr(1,591), ratek(1,591)
c         write(c_out, 308) c_rr(1,592), ratek(1,592)
c307      format('IAP   ->  ev_IAP : rr, ratek= ', 4(1pe10.3))
c308      format('ev_IAP->     IAP : rr, ratek= ', 4(1pe10.3))
         

c      endif   ! end Sandy's additional TEST WRITE

c ORIGINAL TEST WRITE MASTER IF 

c      if( 
c     $ (nsec_jan1.eq.16956000.or.
c     $  nsec_jan1.eq.16959600).and.

c OPTION FOR NO TEST CHEM WRITE
c     &         (il.eq.-305)        

c OPTION FOR SPECIFIC GRIDS 
c   NOTE - Coordinate this with TEST CHEM output  in qk_update.F 
c ANTARCTIC 12-7-09 CRASH
c     &         (il.eq. 3.and. ij.eq. 2.and.(ik.le.3            ) )
c     &         .or.c_xcout(1,101).gt.1.0E+15

c     &         (il.eq.67.and. ij.eq.2.and.(ik.eq.1
c     $          .or.ik.eq.2.or.ik.eq.3            ) )

c     &    (il.eq.3.and. ij.eq.2 .and.ik.eq.1)
c OHIO
c     &         (il.eq.56.and. ij.eq.33.and.(ik.eq.1           ) )
c  NORTH ATLANTIC
c     $  .or. (il.eq.60.and.ij.eq.33.and.ik.eq.1)
c ANTARCTICA  il=longitude 
c     &        (il.eq.66.and. ij.eq. 2.and.ik.eq.1)
c    &         (il.eq.56.and. ij.eq.33.and.(ik.eq.1.or.ik.eq.14) )
c    &         (il.eq.59.and. ij.eq. 6.and.(ik.eq.1.or.ik.eq.14) )
c     &   .or.  (il.eq.36.and. ij.eq. 2.and.(ik.eq.1.or.ik.eq.14) )
c MID-LATITUDE SOUTHERN HEMISPHERE
c    &   .or. (il.eq.66.and. ij.eq. 9.and.ik.eq.1)
c ANTARCTICA  il=longitude 
c     &   .or. (il.eq.66.and. ij.eq. 2.and.ik.eq.1)

c     &                                           )then
c         write(c_out, 201) il, ij, ik
c201      format(/'TEST CHEMICAL OUTPUT: LON, LAT, SIG il,ij,ik= ',3i3)
c         write(c_out,*) nsec_jan1
c          write(c_out, 203) c_temp(1), c_h2oliq(1), c_h2ogas(1)
c203      format(/,'  temp, CLWC, WATV =',(3(1pe10.3)) )
c                 call chemwrite(1,
c     &      c_xcin,  c_xcout,
c     &      xcfinr,  rateh, rhdif, ratek, rateq, rateqq,
c     &       c_rr, c_rp, c_rl
c     &       xrr, rpro, rloss)
c     &     , c_h2oliq, c_temp, c_iter 
c     &     )  

c      endif

c-----------Guangxing Lin -11-27-2011-----------------
c-    
            else if(.not.do_trop_chem(il,ij,ik)) then
               
              do iq = 1, c_nreac
                ratek_imp(il,ij,ik,iq) = 0.0 
                ratek_cld_imp(il,ij,ik,iq)= 0.0
              end do
c-----------Guangxing Lin -11-27-2011-----------------

            end if
          end do ! il
        end do   ! ij
      end do     ! ik

 
!=========debug======Guangxing Lin
      end if

      Deallocate(c_xcin)
      Deallocate(c_xcout)
      Deallocate(c_xcav)
      Deallocate(c_xcemit)
c     Deallocate(c_xcwdep)
      Deallocate(c_rp)
      Deallocate(c_rl)
      Deallocate(c_rr)

      Deallocate(xcfinr)
      Deallocate(ratek)
      Deallocate(rateh)
      Deallocate(rateq)
      Deallocate(rateqq)
      Deallocate(rhdif)
      Deallocate(egasaq)
c 

c Deallocate variables moved from quadchem
      Deallocate (  xc)
c     Deallocate (  xcfinr)
      Deallocate (  xclastq)
      Deallocate (  xcoutcld)
      Deallocate (  c_rrcld)
      Deallocate (  c_rraer)
c
      Deallocate (cpm )
      Deallocate (cpro )
      Deallocate (history )
      Deallocate (geomavg )
      Deallocate (rlm )
      Deallocate (rlmulti )
      Deallocate (rloss )
      Deallocate (rloss1 )
      Deallocate (rlpair )
      Deallocate (rpm )
      Deallocate (rpmulti )
      Deallocate (rppair )
      Deallocate (rpro )
      Deallocate (rpro1 )
      Deallocate (rrp )
      Deallocate (rrl )
      Deallocate (xrm )
      Deallocate (xrp )
      Deallocate (xrppair )
      Deallocate (xrr )
      Deallocate (xrrm )
      Deallocate (xrrpair )

      Deallocate (qaerwat )
      Deallocate (wetrad_aer )
      Deallocate (rh )

      Deallocate (AX)
      Deallocate (BX)
      Deallocate (XX)


      if (pr_diag) then
        Write (6,*) 'Finished Do_Quad_Solver called by ', loc_proc
c                   if(
c     &     i1.le.66.and.i2.ge.66
c     &              .and.ju1.le.2.and.j2 .ge.2) then

c        Write (43,666) rate_total_before,rate_total 
c         write(43,666)ratek_imp(66,2,15,644),ratek_cld_imp(66,2,15,644)        
c         write(6,666)ratek_imp(66,2,15,644),ratek_cld_imp(66,2,15,644)        
c        Write (6,666) rate_total_before,rate_total 
c666      format(/,' after Quad_solver,qk, qk_cld   =',((2pe10.3)) )
c         end if
      end if

      return

      end

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Strat_OH
c
c DESCRIPTION
c   This routine updates the concentrations in the stratosphre.
c
c ARGUMENTS
c   chemintv    : chemistry time step (s)
c   const       : species concentration, known at zone centers (mixing ratio)
c   do_trop_chem: do tropospheric chemistry
c   ratek_imp   : thermal rate constants (units vary)
c
c-----------------------------------------------------------------------------
      subroutine Strat_OH
     &   (chemintv, const_nb,
     &    ratek_imp, do_trop_chem)

      use h_imp_dims
      use h_setkin_par
      use h_setkin_synspc
      use h_sulfquad

      implicit none

c#     include "imp_dims.h"
c#     include "setkin_par.h"
c#     include "setkin_synspc.h"
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c#     include "sulfquad.h"
c ** End of modifications by Luis Olcese, UM
c ********************************************

c     --------------------------------
c     IMPACT-related values.
c     --------------------------------

      logical :: do_trop_chem(i1:i2, ju1:j2, k1:k2)

      real*8  :: chemintv
      real*8  :: const_nb (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: ratek_imp(i1:i2, ju1:j2, k1:k2, num_qks)

c     ----------------------
c     Variable declarations.
c     ----------------------

      logical, save :: first = .true.

      integer :: il, ij, ik, ic

c     ----------------
c     Begin execution.
c     ----------------
 
      if (pr_diag) then
        Write (6,*) 'Strat_OH called by ', loc_proc
        Write (6,*) 'num_active, IOH, ISYNOZ=',num_active, IOH,ISYNOZ
      end if

c DEBUG
c      write(6,*) 'StratOH1->ju1 j2 i1 i2 k1 k2=',
c    *           ju1, j2, i1,i2,k1,k2
c      write(6,*) 'StratOH2->num_active=        ',num_active
c      write(6,*) 'do_trop_chem i1 ju1 k1 ='
c      write(6,*) i1,ju1,k1
c      write(6,*) do_trop_chem(i1,ju1,k1)
c      write(6,*) i2,j2,k2
c      write(6,*) do_trop_chem(i2,j2,k2)


c ----------------------------------------------------------------
c The effects of stratospheric chemistry are simulated by imposing 
c OH reactions with many species above the tropopause.  
c ----------------------------------------------------------------

      do ij = ju1, j2
        do il = i1, i2
          do ik = k1, k2
            if (.not. do_trop_chem(il,ij,ik)) then
croy: why use num_active = 88 instead of 90??
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include the quadchem/sulfur chemistry
c              do ic = 1, num_active
              do ic = div_const + 1, num_active
c ** End of modifications by Luis Olcese, UM
c ********************************************
                if (strat_loss(ic) /= 0) then
!
!Guangxing Lin ---debug only
                  const_nb(il,ij,ik,ic) =
     &            const_nb(il,ij,ik,ic) -
     &            (const_nb(il,ij,ik,ic) * const_nb(il,ij,ik,IOH) *
     &             ratek_imp(il,ij,ik,strat_loss(ic)) * chemintv)

                  if (const_nb(il,ij,ik,ic) < 0.0d0) then
                    const_nb(il,ij,ik,ic) = 1.0d-30
c                    Write (6,*) 'Minus concentration ', il,ij,ik,ic
                  end if
                end if
              end do
            end if
          end do
        end do
      end do

      if (pr_diag) then
        Write (6,*) 'Finished Strat_OH called by ', loc_proc
      end if

      return

      end

c-----------------------------------------------------------------------------
c
c ROUTINE
c calc_aero_wateruptake
c 
c Purpose
c calcaulte the aerosol water and wet radius
c 
c Author
c Guangxing Lin 09-2011
c
c-------------------------------------------------------------------------------

      subroutine calc_aero_wateruptake (const_nb, temp_imp, press3c,
     &           qaerwat, wetrad_aer, rh)

      use h_imp_dims
      use h_imp_control
      use h_setkin_par
      implicit none

c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "setkin_par.h"

c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8  :: const_nb (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: temp_imp  (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: press3c(ilo:ihi, julo:jhi, k1:k2)
      real*8  :: qaerwat (i1:i2, ju1:j2, k1:k2)
      real*8  :: wetrad_aer (i1:i2, ju1:j2, k1:k2)
      real*8  :: rh (i1:i2, ju1:j2, k1:k2)

c     ----------------------
c     variable declarations.
c     ----------------------

      real*8  :: air_density(i1:i2, ju1:j2, k1:k2)
      real*8  :: num_so41 (i1:i2, ju1:j2, k1:k2)
      real*8  :: num_so42 (i1:i2, ju1:j2, k1:k2)
      real*8  :: num_so43 (i1:i2, ju1:j2, k1:k2)
      real*8  :: mas_so41 (i1:i2, ju1:j2, k1:k2)
      real*8  :: mas_so42 (i1:i2, ju1:j2, k1:k2)
      real*8  :: mas_so43 (i1:i2, ju1:j2, k1:k2)
      real*8  :: rv_so41 (i1:i2, ju1:j2, k1:k2)
      real*8  :: rv_so42 (i1:i2, ju1:j2, k1:k2)
      real*8  :: rv_so43 (i1:i2, ju1:j2, k1:k2)
      real*8  :: rm_so41 (i1:i2, ju1:j2, k1:k2)
      real*8  :: rm_so42 (i1:i2, ju1:j2, k1:k2)
      real*8  :: rm_so43 (i1:i2, ju1:j2, k1:k2)
      real*8  :: surf_aer (i1:i2, ju1:j2, k1:k2)
      real*8  :: vol_aer (i1:i2, ju1:j2, k1:k2)

      integer :: il,ij,ik,m
c     -----------------------
c     Parameter declarations.
c     -----------------------
      real*8, parameter :: pi = 3.1415926d0
      real*8, parameter :: pi43 = 3.1415926d0*4.0d0/3.0d0
      real*8, parameter :: rhosul = 1.769d3   !kg/m^3
c      real*8, parameter :: sigmod01 = 1.59d0 !sig of so4n1
c      real*8, parameter :: sigmod02 = 1.514d0 !sig of so4n2
c      real*8, parameter :: sigmod03 = 1.77623d0 !sig of so4n3
      real*8, parameter :: sigmod01 = 1.514d0 !sig of so4n1
      real*8, parameter :: sigmod02 = 1.77623d0 !sig of so4n2
      real*8, parameter :: sigmod03 = 1.77623d0 !sig of so4n3
      real*8, parameter :: third = 1.0d0/3.0d0

      real*8, parameter :: density_water=1.0d6  ! units(g/m^3)
      integer,parameter :: ntot_amode=2
      
      real*8 :: hygro(ntot_amode)
      real*8 :: dryrad(ntot_amode)
      real*8 :: wetrad(ntot_amode)
      real*8 :: dryvol(ntot_amode)
      real*8 :: wetvol(ntot_amode)
      real*8 :: wtrvol(ntot_amode)
      real*8 :: rhcrystal_amode(ntot_amode)
      real*8 :: rhdeliques_amode(ntot_amode)
      real*8 :: hystfac(ntot_amode)
      data hygro/0.51,0.51/
      data rhdeliques_amode/0.8,0.8/
      data rhcrystal_amode/0.4,0.4/
c      data hygro/0.51,0.51,0.51/
c      data rhdeliques_amode/0.8,0.8,0.8/
c      data rhcrystal_amode/0.4,0.4,0.4/
      real*8 :: rh_temp 
c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'calc_aero_wateruptake called by ', loc_proc
        Write (43,*) 'calc_aero_wateruptake called by ', loc_proc
      end if
      
      dryrad=0.0
      wetrad=0.0
      dryvol=0.0
      wetvol=0.0
      wtrvol=0.0               

c--------------------------gxlin-11-10-2009---------------------------------
c     Calculate the surface area density based on the mass, number concentration        
c     and size distribution of sulfate.
c----------------------------------------------------------------------------

c     calculate the air density in units kg/m^3
      
c      air_density(:,:,:) = const_nb(:,:,:,imgas_num) *
c     &  1.0d6 * 28.96d0 * 1.0d-3 / 6.022d23
      air_density(:,:,:) = 
     &  1.0d6 * 28.96d0 * 1.0d-3 / 6.022d23


c     get SO4 number concentraion and mass concentration

c     notes: number and mass should be divided by const_nb(:,:,:,imgas_num)
c     because in the Update_Quadchem subroutine, mixing ratio has been 
c     converted to concentration by multiplying const(:,:,:,imgas_num)

      num_so41(:,:,:) =const_nb(:,:,:,ISO4N2)
     $   * air_density(:,:,:)     ! #/m^3
      num_so42(:,:,:) =const_nb(:,:,:,ISO4N3)
     &   * air_density(:,:,:)     ! #/m^3
      num_so43(:,:,:) =const_nb(:,:,:,ISO4N3)
     &   * air_density(:,:,:)     ! #/m^3
      mas_so41(:,:,:) =const_nb(:,:,:,ISO4M2)
     &   * air_density(:,:,:)     ! kg/m^3
      mas_so42(:,:,:) =const_nb(:,:,:,ISO4M3)
     &   * air_density(:,:,:)     ! kg/m^3
      mas_so43(:,:,:) =const_nb(:,:,:,ISO4M3)
     &   * air_density(:,:,:)     ! kg/m^3
  
c    get volume mean radius (m)

      where (num_so41.eq.0.0)

         rv_so41=0.0d0

      elsewhere

         rv_so41(:,:,:) = (3.d0 * mas_so41(:,:,:) / num_so41(:,:,:) /
     &     (4.d0 * pi * rhosul)) ** (1.d0/3.d0)

      end where

      where (num_so42.eq.0.0)

        rv_so42=0.0d0

      elsewhere

        rv_so42(:,:,:) = (3.d0 * mas_so42(:,:,:) / num_so42(:,:,:) /
     &     (4.d0 * pi * rhosul)) ** (1.d0/3.d0)

       end where

       where (num_so43.eq.0.0)

      rv_so43=0.0d0

      elsewhere

      rv_so43(:,:,:) = (3.d0 * mas_so43(:,:,:) / num_so43(:,:,:) /
     &     (4.d0 * pi * rhosul)) ** (1.d0/3.d0)

      end where


c    get mode radius (m)

      rm_so41(:,:,:) = rv_so41(:,:,:) * exp(-3.d0 *
     $   log(sigmod01) ** 2.d0 / 2.d0 )
      rm_so42(:,:,:) = rv_so42(:,:,:) * exp(-3.d0 *
     $  log(sigmod02) ** 2.d0 / 2.d0 )

        rm_so43(:,:,:) = rv_so43(:,:,:) * exp(-3.d0 *
     $  log(sigmod03) ** 2.d0 / 2.d0 )

c
c    change units (m-->cm)

c      rm_so41(:,:,:) = rm_so41(:,:,:) * 1.d2
c      rm_so42(:,:,:) = rm_so42(:,:,:) * 1.d2

c    get SO4 area density (cm^2/cm^3)

c      lbs_sad(:,:,:) = 4.d0 * pi * rm_so41(:,:,:)
c     &   ** 2.d0 * num_so41(:,:,:) * 1.d-6
c     &   +4.d0 * pi * rm_so42(:,:,:) ** 2.d0 * num_so42(:,:,:) * 1.d-6

      do ij = ju1, j2
        do il = i1, i2
          do ik = k1, k2
             do m=1,ntot_amode
               if(m.eq.1) then
                 dryrad(m)= rm_so41(il,ij,ik)
               else if(m.eq.2) then
                 dryrad(m)= rm_so42(il,ij,ik)
               else
                 dryrad(m)= rm_so43(il,ij,ik)
               end if
               
                rh_temp= rh(il,ij,ik)
               call modal_aero_kohler_um(                          
     &                      dryrad(m), hygro(m), rh_temp,        
     &                       wetrad(m), 1, 1)
                                               
              
               wetrad(m)=
     &         max(wetrad(m),dryrad(m))
               dryvol(m)=pi43*dryrad(m)**3.0
c            dgncur_awet(i,k,m) = dgncur_a(i,k,m)*   &
c                                     (wetrad(i,k,m)/dryrad(i,k,m))
               wetvol(m) = pi43*wetrad(m)**3.0
               wetvol(m) = max(wetvol(m),dryvol(m))
               wtrvol(m) = wetvol(m) - dryvol(m)
               wtrvol(m) = max( wtrvol(m), 0.d0 )
 
c     apply simple treatment of deliquesence/crystallization hysteresis
c     for rhcrystal < rh < rhdeliques, aerosol water is a fraction of
c     the "upper curve" value, and the fraction is a linear function of rh
            
            hystfac(m) = 1.0d0/max(1.0d-5,
     &                   (rhdeliques_amode(m)-rhcrystal_amode(m)))
            if (rh(il,ij,ik) < rhcrystal_amode(m)) then
               wetrad(m) = dryrad(m)
               wetvol(m) = dryvol(m)
               wtrvol(m) = 0.d0
            else if (rh(il,ij,ik) < rhdeliques_amode(m)) then
               wtrvol(m) = wtrvol(m)*hystfac(m)  
     &                       *(rh(il,ij,ik) - rhcrystal_amode(m))
               wtrvol(m) = max( wtrvol(m), 0.d0 )
               wetvol(m) = dryvol(m) + wtrvol(m)
               wetrad(m) = (wetvol(m)/pi43)**third
            end if

c            if ( aero_mmr_flag ) then
c               duma = 1.0d0
c            else
c               duma = mwdry/mwh2o
c            end if
c            if(m.eq.1) then
c             qwater = density_water*num_so41(il,ij,ik)*wtrvol(m)*duma
c            else
c             qwater = density_water*num_so42(il,ij,ik)*wtrvol(m)*duma
c            end if    
c             qaerwat(il,ij,ik,m) = qwater

           end do !m

        
           qaerwat(il,ij,ik) = density_water*
     &        (num_so41(il,ij,ik)*wtrvol(1) +
     &         num_so42(il,ij,ik)*wtrvol(2))*1.0d-6    ! 1.0d-6 is for m^-3 to cm^-3

            surf_aer(il,ij,ik) = 4.d0 * pi * (wetrad(1) 
     &        ** 2.d0 * num_so41(il,ij,ik) 
     &        + wetrad(2) ** 2.d0 * num_so42(il,ij,ik) )

            vol_aer(il,ij,ik) = 4.d0/3.d0 * pi * (wetrad(1)
     &        **3.d0 * num_so41(il,ij,ik)
     &        + wetrad(2)**3.d0 * num_so42(il,ij,ik))

     
             if(surf_aer(il,ij,ik).eq.0.0) then
               wetrad_aer(il,ij,ik)=0.0d0
             else
            wetrad_aer(il,ij,ik) = 
     &        vol_aer(il,ij,ik)/surf_aer(il,ij,ik)*3.d0

             end if
            if((il.eq.56.and.ij.eq.33.and.ik.eq.4)) then
         write(6,*) 'IN CACL_aerowater-steping: il, ij, ik=', il,ij,ik
c         write(6,*) ' num_so41,num_so42,wtrvol1,wtrvol2,airdensity  '
c         write(6,333) num_so41(il,ij,ik),  num_so42(il,ij,ik),
c     &     wtrvol(1), wtrvol(2), air_density(il,ij,ik)
         write(6,*) 'dryrad_1,wetrad_1,dryrad_2,wetrad_2 '
         write(6,333) dryrad(1), wetrad(1),dryrad(2),wetrad(2)

333      format(4(1pe10.3)) 
       endif

          end do !ik
         end do ! ij
        end do !  il

      return
      end


      
c-----------------------------------------------------------------------------
c
c ROUTINE
c modal_aero_kohler
c 
c PURPOSE
c calculates equlibrium radius r of haze droplets as function of
c dry particle mass and relative humidity s using kohler solution
c given in pruppacher and klett (eqn 6-35)
c
c for multiple aerosol types, assumes an internal mixture of aerosols
c
c Author
c Guangxing Lin---10-2011
c
c-------------------------------------------------------------------------------
      subroutine modal_aero_kohler_um(  
     &     rdry_in, hygro, s, rwet_out, im, imx )

      use shr_kind_mod,     only: r8 => shr_kind_r8

      implicit none

c arguments
      integer :: im         ! number of grid points to be processed
      integer :: imx        ! dimensioned number of grid points
      real*8 :: rdry_in(imx)    ! aerosol dry radius (m)
      real*8 :: hygro(imx)      ! aerosol volume-mean hygroscopicity (--)
      real*8 :: s(imx)          ! relative humidity (1 = saturated)
      real*8 :: rwet_out(imx)   ! aerosol wet radius (m)

c local variables
      integer, parameter :: imax=200
      integer :: i, n, nsol

      real*8 :: a, b
      real*8 :: p40(imax),p41(imax),p42(imax),p43(imax) ! coefficients of polynomial
      real*8 :: p30(imax),p31(imax),p32(imax) ! coefficients of polynomial
      real*8 :: p
      real*8 :: r3, r4
      real*8 :: r(imx)        ! wet radius (microns)
      real*8 :: rdry(imax)    ! radius of dry particle (microns)
      real*8 :: ss            ! relative humidity (1 = saturated)
      real*8 :: slog(imax)    ! log relative humidity
      real*8 :: vol(imax)     ! total volume of particle (microns**3)
      real*8 :: xi, xr

      complex(r8) :: cx4(4,imax),cx3(3,imax)

      real*8, parameter :: eps = 1.e-4_r8
      real*8, parameter :: mw = 18._r8
      real*8, parameter :: pi = 3.14159_r8
      real*8, parameter :: rhow = 1._r8
      real*8, parameter :: surften = 76._r8
      real*8, parameter :: tair = 273._r8
      real*8, parameter :: third = 1./3._r8
      real*8, parameter :: ugascon = 8.3e7_r8


!     effect of organics on surface tension is neglected
      a=2.e4_r8*mw*surften/(ugascon*tair*rhow)

      do i=1,im
           rdry(i) = rdry_in(i)*1.0e6_r8   ! convert (m) to (microns)
           vol(i) = rdry(i)**3          ! vol is r**3, not volume
           b = vol(i)*hygro(i)

!          quartic
           ss=min(s(i),1._r8-eps)
           ss=max(ss,1.e-10_r8)
           slog(i)=log(ss)
           p43(i)=-a/slog(i)
           p42(i)=0._r8
           p41(i)=b/slog(i)-vol(i)
           p40(i)=a*vol(i)/slog(i)
!          cubic for rh=1
           p32(i)=0._r8
           p31(i)=-b/a
           p30(i)=-vol(i)
      end do


       do 100 i=1,im

c       if(vol(i).le.1.e-20)then
        if(vol(i).le.1.e-12_r8)then
           r(i)=rdry(i)
           go to 100
        endif

        p=abs(p31(i))/(rdry(i)*rdry(i))
        if(p.lt.eps)then
!          approximate solution for small particles
           r(i)=rdry(i)*(1._r8+p*third/(1._r8-slog(i)*rdry(i)/a))
        else
           call makoh_quartic(cx4(1,i),p43(i),p42(i),p41(i),p40(i),1)
!          find smallest real(r8) solution
           r(i)=1000._r8*rdry(i)
           nsol=0
           do n=1,4
              xr=real(cx4(n,i))
              xi=imag(cx4(n,i))
              if(abs(xi).gt.abs(xr)*eps) cycle  
              if(xr.gt.r(i)) cycle  
              if(xr.lt.rdry(i)*(1._r8-eps)) cycle  
              if(xr.ne.xr) cycle  
              r(i)=xr
              nsol=n
           end do  
           if(nsol.eq.0)then
c              write(6,*)   
c     &          'ccm kohlerc - no real(r8) solution found (quartic)'
c              write(6,*)'roots =', (cx4(n,i),n=1,4)
c              write(6,*)'p0-p3 =', p40(i), p41(i), p42(i), p43(i)
c              write(6,*)'rh=',s(i)
c              write(6,*)'setting radius to dry radius=',rdry(i)
              r(i)=rdry(i)
!             stop
           endif
        endif

        if(s(i).gt.1._r8-eps)then
!          save quartic solution at s=1-eps
           r4=r(i)
!          cubic for rh=1
           p=abs(p31(i))/(rdry(i)*rdry(i))
           if(p.lt.eps)then
              r(i)=rdry(i)*(1._r8+p*third)
           else
              call makoh_cubic(cx3,p32,p31,p30,im)
!             find smallest real(r8) solution
              r(i)=1000._r8*rdry(i)
              nsol=0
              do n=1,3
                 xr=real(cx3(n,i))
                 xi=imag(cx3(n,i))
                 if(abs(xi).gt.abs(xr)*eps) cycle  
                 if(xr.gt.r(i)) cycle  
                 if(xr.lt.rdry(i)*(1._r8-eps)) cycle  
                 if(xr.ne.xr) cycle  
                 r(i)=xr
                 nsol=n
              end do  
              if(nsol.eq.0)then
c                 write(6,*)   
c     &             'ccm kohlerc - no real(r8) solution found (cubic)'
c                 write(6,*)'roots =', (cx3(n,i),n=1,3)
c                 write(6,*)'p0-p2 =', p30(i), p31(i), p32(i)
c                 write(6,*)'rh=',s(i)
c                 write(6,*)'setting radius to dry radius=',rdry(i)
                 r(i)=rdry(i)
!                stop
              endif
           endif
           r3=r(i)
!          now interpolate between quartic, cubic solutions
           r(i)=(r4*(1._r8-s(i))+r3*(s(i)-1._r8+eps))/eps
        endif

  100 continue

! bound and convert from microns to m
      do i=1,im
         r(i) = min(r(i),30._r8) ! upper bound based on 1 day lifetime
         rwet_out(i) = r(i)*1.e-6_r8
      end do

      return
      end subroutine modal_aero_kohler_um


c-----------------------------------------------------------------------

       subroutine makoh_cubic( cx, p2, p1, p0, im )
!
!     solves  x**3 + p2 x**2 + p1 x + p0 = 0
!     where p0, p1, p2 are real
!
      use shr_kind_mod,     only: r8 => shr_kind_r8
      integer, parameter :: imx=200
      integer :: im
      real*8 :: p0(imx), p1(imx), p2(imx)
      complex(r8) :: cx(3,imx)

      integer :: i
      real*8 :: eps, q(imx), r(imx), sqrt3, third
      complex(r8) :: ci, cq, crad(imx), cw, cwsq, cy(imx), cz(imx)

      save eps
      data eps/1.e-20_r8/

      third=1._r8/3._r8
      ci=dcmplx(0._r8,1._r8)
      sqrt3=sqrt(3._r8)
      cw=0.5_r8*(-1+ci*sqrt3)
      cwsq=0.5_r8*(-1-ci*sqrt3)

      do i=1,im
      if(p1(i).eq.0._r8)then
!        completely insoluble particle
         cx(1,i)=(-p0(i))**third
         cx(2,i)=cx(1,i)
         cx(3,i)=cx(1,i)
      else
         q(i)=p1(i)/3._r8
         r(i)=p0(i)/2._r8
         crad(i)=r(i)*r(i)+q(i)*q(i)*q(i)
         crad(i)=sqrt(crad(i))

         cy(i)=r(i)-crad(i)
         if (abs(cy(i)).gt.eps) cy(i)=cy(i)**third
         cq=q(i)
         cz(i)=-cq/cy(i)

         cx(1,i)=-cy(i)-cz(i)
         cx(2,i)=-cw*cy(i)-cwsq*cz(i)
         cx(3,i)=-cwsq*cy(i)-cw*cz(i)
      endif
      enddo

      return
      end subroutine makoh_cubic


!-----------------------------------------------------------------------
         subroutine makoh_quartic( cx, p3, p2, p1, p0, im )

         use shr_kind_mod,     only: r8 => shr_kind_r8
!     solves x**4 + p3 x**3 + p2 x**2 + p1 x + p0 = 0
!     where p0, p1, p2, p3 are real
!
      integer, parameter :: imx=200
      integer :: im
      real*8 :: p0(imx), p1(imx), p2(imx), p3(imx)
      complex(r8) :: cx(4,imx)

      integer :: i
      real*8 :: third, q(imx), r(imx)
      complex(r8) :: cb(imx), cb0(imx), cb1(imx),   
     &                crad(imx), cy(imx), czero


      czero=cmplx(0.0_r8,0.0_r8)
      third=1._r8/3._r8

      do 10 i=1,im

      q(i)=-p2(i)*p2(i)/36._r8+(p3(i)*p1(i)-4*p0(i))/12._r8
      r(i)=-(p2(i)/6)**3+p2(i)*(p3(i)*p1(i)-4*p0(i))/48._r8   
     &   +(4*p0(i)*p2(i)-p0(i)*p3(i)*p3(i)-p1(i)*p1(i))/16

      crad(i)=r(i)*r(i)+q(i)*q(i)*q(i)
        crad(i)=sqrt(crad(i))

      cb(i)=r(i)-crad(i)
      if(cb(i).eq.czero)then
!        insoluble particle
         cx(1,i)=(-p1(i))**third
         cx(2,i)=cx(1,i)
         cx(3,i)=cx(1,i)
         cx(4,i)=cx(1,i)
      else
         cb(i)=cb(i)**third

         cy(i)=-cb(i)+q(i)/cb(i)+p2(i)/6

         cb0(i)=sqrt(cy(i)*cy(i)-p0(i))
         cb1(i)=(p3(i)*cy(i)-p1(i))/(2*cb0(i))

         cb(i)=p3(i)/2+cb1(i)
         crad(i)=cb(i)*cb(i)-4*(cy(i)+cb0(i))
         crad(i)=sqrt(crad(i))
         cx(1,i)=(-cb(i)+crad(i))/2._r8
         cx(2,i)=(-cb(i)-crad(i))/2._r8

         cb(i)=p3(i)/2-cb1(i)
         crad(i)=cb(i)*cb(i)-4*(cy(i)-cb0(i))
         crad(i)=sqrt(crad(i))
         cx(3,i)=(-cb(i)+crad(i))/2._r8
         cx(4,i)=(-cb(i)-crad(i))/2._r8
       endif
   10 continue

      return
      end subroutine makoh_quartic

!----------------------------------------------------------------------

