c=============================================================================
c
c $Id: strat_update.F,v 1.4 2002/08/27 21:11:10 akinorii Exp $
c
c CODE DEVELOPER
c   Akinori Ito, U of M
c
c FILE
c   strat_update.F
c
c ROUTINES
c   Do_Strat_Set
c
c=============================================================================
 
c-----------------------------------------------------------------------------
c DEBUG 907 writes added. (if pr_diag)
c
c ROUTINE
c   Do_Strat_Set
c
c DESCRIPTION
c   This routine updates the concentrations in the stratosphre.
c
c ARGUMENTS
c   const        : species concentration, known at zone centers (mixing ratio)
c   do_trop_chem : do tropospheric chemistry
c   londeg       : longitude (deg)
c   latdeg       : latitude  (deg)
c
c-----------------------------------------------------------------------------
      subroutine Do_Strat_Set
     &   (const, do_trop_chem, londeg, latdeg)

      use h_imp_dims
      use h_imp_control
      use h_setkin_par

      implicit none

c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "setkin_par.h"

c     --------------------------------
c     IMPACT-related values.
c     --------------------------------

      logical :: def_trop
      logical :: do_trop_chem(i1:i2, ju1:j2, k1:k2)

!     real*8  :: londeg (i1_gl:i2_gl)
      real*8  :: londeg (ju1_gl:j2_gl) !Guangxing Lin
      real*8  :: latdeg (ju1_gl:j2_gl)
      real*8  :: const(i1:i2, ju1:j2, k1:k2, num_species)

c     ----------------------
c     Variable declarations.
c     ----------------------

      logical, save :: first = .true.

      integer :: il, ij, ik, ic
      real*8, allocatable  :: oh_scaler(:,:)

c     ----------------
c     Begin execution.
c     ----------------
 
      if (pr_diag) then
        Write (6,*) 'Do_Strat_Set called by ', loc_proc
      end if

      Allocate (oh_scaler(i1:i2, ju1:j2))

      oh_scaler = 0.0d0

c --------------------------------------------------------------
c Tropospheric chemistry is applied below the 120-ppbv isopleth.
c
c Concentrations of ozone are tied
c to observations above the tropopause 
c for the calculation of photolysis rates.
c
c Concentrations of OH above the tropopause are 
c derived from 2-D GRANTOUR.
c --------------------------------------------------------------

c DEBUG 907
      if (pr_diag) then
        write(6 ,*)'START  STRAT_SET: do_synoz =', do_synoz
        write(6 ,*) ju1,j2, i1, i2, k1, k2
        write(6 ,*) 'isynoz_num =', isynoz_num
      endif

      if (do_synoz) then
        do ij = ju1, j2
          do il = i1, i2
            def_trop = .true.
            do ik = k1, k2
              if (def_trop) then
                if (const(il,ij,ik,isynoz_num)
     &            > synoz_threshold) then
                  def_trop = .false.
                  do_trop_chem(il,ij,ik) = .false.
                end if
              else
                do_trop_chem(il,ij,ik) = .false.
              end if
c DEBUG 907
              if(pr_diag.and.il.eq.41.and.ij.eq.35) then
                write(6 ,*) 'STRAT_SET: il, ij, ik, do_trop=', 
     &           il, ij, ik ,  do_trop_chem(il, ij, ik)
                write(6 ,*) ' threshhold, O3 =',
     &           synoz_threshold, 
     &             const(il,ij,ik,isynoz_num)
              endif    ! debug

            end do
          end do
        end do

c       --------------------------------------------------------
c       Calculate the scaling factor for OH diurnal variations.
c       --------------------------------------------------------

c       =====================
        call Scale_OH
c       =====================
     &  (nymd, nhms, londeg, latdeg, oh_scaler) 

        do ij = ju1, j2
          do il = i1, i2
            do ik = k1, k2
              if (do_trop_chem(il,ij,ik)) then
                const(il,ij,ik,IO3S) = const(il,ij,ik,IO3)
              else
!Guangxing Lin--debug only
                const(il,ij,ik,IOH) =
     &          const(il,ij,ik,IOHS) * oh_scaler(il,ij)
              end if
            end do
          end do
        end do
      end if

      Deallocate (oh_scaler)

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Scale_OH
c
c Original code
c   Scale_HOx_NO3_Qj from Xiaohong Liu
c
c DESCRIPTION
c   This routine computes the scaling factor for OH based on
c   OH(instantaneous) = OH(max) * cos(theta)
c   where cos(theta) is cosine of solar zenith angle.
c
c   thus, OH(monthly_ave) * 24 (hr) = OH(max) * Sum (cos(theta) * dt)
c                                     ( t (hr) = triz -> tset)
c
c   OH(max) = OH(monthly_ave) * 24 (hr) / (Sum (cos(theta) * dt) )
c
c   so, oh_scaler = 24 / (Sum (cos(theta) * dt) ) * cos(theta)
c
c-----------------------------------------------------------------------------

      subroutine Scale_OH
     &  (nymd, nhms, londeg, latdeg, oh_scaler)

      use h_imp_time_constants
      use h_imp_dims
      implicit none

c#     include "imp_time_constants.h"
c#     include "imp_dims.h"

c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: nymd
      integer :: nhms

!      real*8  :: londeg (i1_gl:i2_gl)
      real*8  :: londeg (ju1_gl:j2_gl) !Guangxing
      real*8  :: latdeg (ju1_gl:j2_gl)
      real*8  :: oh_scaler(i1:i2, ju1:j2)

c     ----------------------
c     Variable declarations.
c     ----------------------


      integer :: il, ij, it
      integer :: idumyear, idummin, idumsec
      integer :: im, id, ih
      integer :: nday_jan1
      integer :: nsec_jan1
      integer :: nt

      real*8 :: tloc
      real*8 :: stime_loc
      real*8 :: days
      real*8 :: decl     ! solar declination
      real*8 :: rdistsq  ! solar distance correction
      real*8 :: time
      real*8 :: triz(ju1:j2)
      real*8 :: tset(ju1:j2)
      real*8 :: tot_cossza(ju1:j2)
      real*8, save :: lon_dummy(1) = 0.0d0
      real*8 :: cossza(1)
      real*8 :: csza(i1:i2)

c     ----------------
c     Begin execution.
c     ----------------

      csza(:) = 0.0d0


c     ====================
      call Split_Time_Flds
c     ====================
     &  (nymd, idumyear, im, id)

c     ====================
      call Split_Time_Flds
c     ====================
     &  (nhms, ih, idummin, idumsec)

c     =================
      call Get_Day_Jan1
c     =================
     &  (nday_jan1, nymd)

c     Calculate the sun rize and sun set time (local time: hr)

      do ij = ju1, j2

c       ==============
        call SunRizSet_Quad
c       ==============
     &    (im, id, latdeg(ij), triz(ij), tset(ij))

      end do


      nt = 100       ! divide one-day (24 hr) into 100 time intervals
      tot_cossza(:) = 0.0d0

c     =============
      do it = 0, nt
c     =============

        tloc = it * 24.0d0/nt      ! hr
        stime_loc = nday_jan1 * SECPDY + tloc * SECPHR

        days = stime_loc / SECPDY

c       ============
        call Solrpos
c       ============
     &    (days, decl, rdistsq)


c       ===============
        do ij = ju1, j2
c       ===============

c         ---------------------------------------------
c         Calculate solar zenith angle at this latitude
c         ---------------------------------------------

c         ===========
          call Solrza
c         ===========
     &      (days, decl, latdeg(ij), lon_dummy, 1, cossza)


          if(tloc > triz(ij) .and. tloc < tset(ij)) then

            tot_cossza(ij) = tot_cossza(ij) +
     &                       cossza(1) * 24.0d0/nt

          end if

c       ======
        end do
c       ======

c     ======
      end do
c     ======


c     =================
      call Get_Sec_Jan1
c     =================
     &  (nsec_jan1, nymd, nhms)

      time = nsec_jan1
      days = time / SECPDY

c     ============
      call Solrpos
c     ============
     &  (days, decl, rdistsq)


c     ===============
      do ij = ju1, j2
c     ===============

c       -----------------------------------------------------
c       Calculate solar zenith angle at this latitude for all
c       longitudes.
c       -----------------------------------------------------
c       ===========
        call Solrza
c       ===========
     &    (days, decl, latdeg(ij), londeg(ij), ilong, csza)


        do il = i1, i2

          tloc = ih + londeg(ij) / 360.d0 * 24.0d0  !Guangxing Lin
!change il to ij for londeg
          if(tloc > 24.0d0) tloc = tloc - 24.0d0

          if(tloc > triz(ij) .and. tloc < tset(ij)) then
            oh_scaler(il,ij) = 24.0d0 /
     &                             tot_cossza(ij) * csza(il) 
          else
            oh_scaler(il,ij) = 1.0d-5
          end if

c         -----------------------
c         oh_scaler must be plus.
c         -----------------------
          if (oh_scaler(il,ij) < 1.0d-5) then
            oh_scaler(il,ij) = 1.0d-5
          end if
        end do

c     ======
      end do
c     ======


      return

      end

CLL Subroutine: SunRizSet.f--------------------------------------------
CLL Purpose: Compute the local time of sunrise and sunset given
CLL          latitude and date.
CLL
CLL Author: Wenwei Pan
CLL         Center for Meteorology and Oceanography
CLL         Center for Global Change Science
CLL         Department of Earth, Atmospheric and Planetary Sciences
CLL
CLL Version: 1.0
CLL Date: April 3, 1997
CLL
CLL Tested under compiler: f77 V4.0
CLL Tested under OS version: SunOS 5.5.1 Generic sun4u sparc SUNW,Ultra-1
CLL
CLL Code modification history:
CLL Version Date
CLL---------------------------------------------------------------------
!=======Guangxing Lin   commemnt out below, because it is defined in
!sulfchem_solver.F 
      subroutine SunRizSet_Quad(month,iday,rlat,triz,tset)

      implicit none

!C Input variables.
      integer
     &     month,
     &     iday
      real*8 ::
     &     rlat           !Latitude in degrees.

!C Output variables.
      real*8 :: triz,tset      !local time of sunrise and sunset(hr)

!C Local variables.
      real*8 ::
     &     fac,
     &     Pi,
     &     x,
     &     xlat,          !Latitude in radians.
     &     phi,cosphi,sinphi,
     &     tphi,costph,sintph,
     &     ttphi,costtp,sinttp,
     &     delta,         !Solar inclination angle in radians.
     &     h              !Hour angle at sunrise and sunset in radians.

C*----------------------------------------------------------------------

      FAC = .01745d0
      PI = 3.1415926d0
      X = FLOAT((MONTH-1)*30+IDAY+MONTH/2)
      XLAT = RLAT*FAC
      PHI = 2.0d0*3.14159d0*X/365.0d0
      TPHI = PHI*2.0d0
      TTPHI = PHI*3.0d0
      COSPHI = COS(PHI)
      SINPHI = SIN(PHI)
      COSTPH = COS(TPHI)
      SINTPH = SIN(TPHI)
      COSTTP = COS(TTPHI)
      SINTTP = SIN(TTPHI)
      DELTA = .006918d0 - 0.399912d0*COSPHI + 0.070257d0*SINPHI -
     +  0.006758d0*COSTPH + 0.000907d0*SINTPH - 0.002697d0*COSTTP +
     +  0.00148d0*SINTTP
      IF(DELTA.GE.0.0d0)THEN                   !SUMMER
         IF(XLAT.GE.(PI/2.0d0-DELTA))THEN      !NO SUNSET
            H=PI
         ELSEIF(XLAT.LE.(DELTA-PI/2.0d0))THEN  !NO SUNRISE
            H=0.0d0
         ELSE
            H=ACOS(-TAN(XLAT)*TAN(DELTA))
         ENDIF
      ELSE
         IF(XLAT.GE.(PI/2.0d0+DELTA))THEN      !NO SUNRISE
            H=0.0d0
         ELSEIF(XLAT.LE.(-DELTA-PI/2.0d0))THEN !NO SUNSET
            H=PI
         ELSE
            H=ACOS(-TAN(XLAT)*TAN(DELTA))
         ENDIF
      ENDIF
      Triz=12.0d0*(-H+PI)/PI
      Tset=12.0d0*(H+PI)/PI

      return

      end

